@@ -5,6 +5,7 @@
 #include <openssl/bn.h>
 #include <openssl/evp.h>
 #include <openssl/hmac.h>
+#include <openssl/rand.h>
 #include <openssl/pkcs12.h>
 #include <openssl/x509v3.h>
 #if OPENSSL_VERSION_MAJOR >= 3
@@ -33,7 +34,7 @@ ClearErrorOnReturn::~ClearErrorOnReturn() {
   ERR_clear_error();
 }
 
-int ClearErrorOnReturn::peeKError() { return ERR_peek_error(); }
+int ClearErrorOnReturn::peekError() { return ERR_peek_error(); }
 
 MarkPopErrorOnReturn::MarkPopErrorOnReturn(CryptoErrorList* errors) : errors_(errors) {
   ERR_set_mark();
@@ -356,6 +357,35 @@ int PasswordCallback(char* buf, int size, int rwflag, void* u) {
   return -1;
 }
 
+// Algorithm: http://howardhinnant.github.io/date_algorithms.html
+constexpr int days_from_epoch(int y, unsigned m, unsigned d)
+{
+  y -= m <= 2;
+  const int era = (y >= 0 ? y : y - 399) / 400;
+  const unsigned yoe = static_cast<unsigned>(y - era * 400);            // [0, 399]
+  const unsigned doy = (153 * (m + (m > 2 ? -3 : 9)) + 2) / 5 + d - 1;  // [0, 365]
+  const unsigned doe = yoe * 365 + yoe / 4 - yoe / 100 + doy;           // [0, 146096]
+  return era * 146097 + static_cast<int>(doe) - 719468;
+}
+
+// tm must be in UTC
+// using time_t causes problems on 32-bit systems and windows x64.
+int64_t PortableTimeGM(struct tm* t) {
+  int year = t->tm_year + 1900;
+  int month = t->tm_mon;
+  if (month > 11) {
+    year += month / 12;
+    month %= 12;
+  } else if (month < 0) {
+    int years_diff = (11 - month) / 12;
+    year -= years_diff;
+    month += 12 * years_diff;
+  }
+  int days_since_epoch = days_from_epoch(year, month + 1, t->tm_mday);
+
+  return 60 * (60 * (24LL * static_cast<int64_t>(days_since_epoch) + t->tm_hour) + t->tm_min) + t->tm_sec;
+}
+
 // ============================================================================
 // SPKAC
 
@@ -826,6 +856,18 @@ BIOPointer X509View::getValidTo() const {
   return bio;
 }
 
+int64_t X509View::getValidToTime() const {
+  struct tm tp;
+  ASN1_TIME_to_tm(X509_get0_notAfter(cert_), &tp);
+  return PortableTimeGM(&tp);
+}
+
+int64_t X509View::getValidFromTime() const {
+  struct tm tp;
+  ASN1_TIME_to_tm(X509_get0_notBefore(cert_), &tp);
+  return PortableTimeGM(&tp);
+}
+
 DataPointer X509View::getSerialNumber() const {
   ClearErrorOnReturn clearErrorOnReturn;
   if (cert_ == nullptr) return {};
@@ -1384,4 +1426,333 @@ DataPointer pbkdf2(const EVP_MD* md,
   return {};
 }
 
+// ============================================================================
+
+EVPKeyPointer EVPKeyPointer::New() {
+  return EVPKeyPointer(EVP_PKEY_new());
+}
+
+EVPKeyPointer EVPKeyPointer::NewRawPublic(int id, const Buffer<const unsigned char>& data) {
+  if (id == 0) return {};
+  return EVPKeyPointer(EVP_PKEY_new_raw_public_key(id, nullptr, data.data, data.len));
+}
+
+EVPKeyPointer EVPKeyPointer::NewRawPrivate(int id, const Buffer<const unsigned char>& data) {
+  if (id == 0) return {};
+  return EVPKeyPointer(EVP_PKEY_new_raw_private_key(id, nullptr, data.data, data.len));
+}
+
+EVPKeyPointer::EVPKeyPointer(EVP_PKEY* pkey) : pkey_(pkey) {}
+
+EVPKeyPointer::EVPKeyPointer(EVPKeyPointer&& other) noexcept
+    : pkey_(other.release()) {}
+
+EVPKeyPointer& EVPKeyPointer::operator=(EVPKeyPointer&& other) noexcept {
+  if (this == &other) return *this;
+  this->~EVPKeyPointer();
+  return *new (this) EVPKeyPointer(std::move(other));
+}
+
+EVPKeyPointer::~EVPKeyPointer() { reset(); }
+
+void EVPKeyPointer::reset(EVP_PKEY* pkey) {
+  pkey_.reset(pkey);
+}
+
+EVP_PKEY* EVPKeyPointer::release() {
+  return pkey_.release();
+}
+
+int EVPKeyPointer::id(const EVP_PKEY* key) {
+  if (key == nullptr) return 0;
+  return EVP_PKEY_id(key);
+}
+
+int EVPKeyPointer::base_id(const EVP_PKEY* key) {
+  if (key == nullptr) return 0;
+  return EVP_PKEY_base_id(key);
+}
+
+int EVPKeyPointer::id() const {
+  return id(get());
+}
+
+int EVPKeyPointer::base_id() const {
+  return base_id(get());
+}
+
+int EVPKeyPointer::bits() const {
+  if (get() == nullptr) return 0;
+  return EVP_PKEY_bits(get());
+}
+
+size_t EVPKeyPointer::size() const {
+  if (get() == nullptr) return 0;
+  return EVP_PKEY_size(get());
+}
+
+EVPKeyCtxPointer EVPKeyPointer::newCtx() const {
+  if (!pkey_) return {};
+  return EVPKeyCtxPointer(EVP_PKEY_CTX_new(get(), nullptr));
+}
+
+size_t EVPKeyPointer::rawPublicKeySize() const {
+  if (!pkey_) return 0;
+  size_t len = 0;
+  if (EVP_PKEY_get_raw_public_key(get(), nullptr, &len) == 1) return len;
+  return 0;
+}
+
+size_t EVPKeyPointer::rawPrivateKeySize() const {
+  if (!pkey_) return 0;
+  size_t len = 0;
+  if (EVP_PKEY_get_raw_private_key(get(), nullptr, &len) == 1) return len;
+  return 0;
+}
+
+DataPointer EVPKeyPointer::rawPublicKey() const {
+  if (!pkey_) return {};
+  if (auto data = DataPointer::Alloc(rawPublicKeySize())) {
+    const Buffer<unsigned char> buf = data;
+    size_t len = data.size();
+    if (EVP_PKEY_get_raw_public_key(get(),
+            buf.data,
+            &len) != 1) return {};
+    return data;
+  }
+  return {};
+}
+
+DataPointer EVPKeyPointer::rawPrivateKey() const {
+  if (!pkey_) return {};
+  if (auto data = DataPointer::Alloc(rawPrivateKeySize())) {
+    const Buffer<unsigned char> buf = data;
+    size_t len = data.size();
+    if (EVP_PKEY_get_raw_private_key(get(),
+            buf.data,
+            &len) != 1) return {};
+    return data;
+  }
+  return {};
+}
+
+BIOPointer EVPKeyPointer::derPublicKey() const {
+  if (!pkey_) return {};
+  auto bio = BIOPointer::NewMem();
+  if (!bio) return {};
+  if (!i2d_PUBKEY_bio(bio.get(), get())) return {};
+  return bio;
+}
+
+namespace {
+EVPKeyPointer::ParseKeyResult TryParsePublicKeyInner(
+    const BIOPointer& bp,
+    const char* name,
+    auto&& parse) {
+  if (!bp.resetBio()) {
+    return EVPKeyPointer::ParseKeyResult(EVPKeyPointer::PKParseError::FAILED);
+  }
+  unsigned char* der_data;
+  long der_len;
+
+  // This skips surrounding data and decodes PEM to DER.
+  {
+    MarkPopErrorOnReturn mark_pop_error_on_return;
+    if (PEM_bytes_read_bio(&der_data, &der_len, nullptr, name,
+                           bp.get(), nullptr, nullptr) != 1)
+      return EVPKeyPointer::ParseKeyResult(EVPKeyPointer::PKParseError::NOT_RECOGNIZED);
+  }
+  DataPointer data(der_data, der_len);
+
+  // OpenSSL might modify the pointer, so we need to make a copy before parsing.
+  const unsigned char* p = der_data;
+  EVPKeyPointer pkey(parse(&p, der_len));
+  if (!pkey) return EVPKeyPointer::ParseKeyResult(EVPKeyPointer::PKParseError::FAILED);
+  return EVPKeyPointer::ParseKeyResult(std::move(pkey));
+}
+
+constexpr bool IsASN1Sequence(const unsigned char* data, size_t size,
+                              size_t* data_offset, size_t* data_size) {
+  if (size < 2 || data[0] != 0x30)
+    return false;
+
+  if (data[1] & 0x80) {
+    // Long form.
+    size_t n_bytes = data[1] & ~0x80;
+    if (n_bytes + 2 > size || n_bytes > sizeof(size_t))
+      return false;
+    size_t length = 0;
+    for (size_t i = 0; i < n_bytes; i++)
+      length = (length << 8) | data[i + 2];
+    *data_offset = 2 + n_bytes;
+    *data_size = std::min(size - 2 - n_bytes, length);
+  } else {
+    // Short form.
+    *data_offset = 2;
+    *data_size = std::min<size_t>(size - 2, data[1]);
+  }
+
+  return true;
+}
+
+constexpr bool IsEncryptedPrivateKeyInfo(const Buffer<const unsigned char>& buffer) {
+  // Both PrivateKeyInfo and EncryptedPrivateKeyInfo start with a SEQUENCE.
+  if (buffer.len == 0 || buffer.data == nullptr) return false;
+  size_t offset, len;
+  if (!IsASN1Sequence(buffer.data, buffer.len, &offset, &len))
+    return false;
+
+  // A PrivateKeyInfo sequence always starts with an integer whereas an
+  // EncryptedPrivateKeyInfo starts with an AlgorithmIdentifier.
+  return len >= 1 && buffer.data[offset] != 2;
+}
+
+}  // namespace
+
+bool EVPKeyPointer::IsRSAPrivateKey(const Buffer<const unsigned char>& buffer) {
+  // Both RSAPrivateKey and RSAPublicKey structures start with a SEQUENCE.
+  size_t offset, len;
+  if (!IsASN1Sequence(buffer.data, buffer.len, &offset, &len))
+    return false;
+
+  // An RSAPrivateKey sequence always starts with a single-byte integer whose
+  // value is either 0 or 1, whereas an RSAPublicKey starts with the modulus
+  // (which is the product of two primes and therefore at least 4), so we can
+  // decide the type of the structure based on the first three bytes of the
+  // sequence.
+  return len >= 3 &&
+         buffer.data[offset] == 2 &&
+         buffer.data[offset + 1] == 1 &&
+         !(buffer.data[offset + 2] & 0xfe);
+}
+
+EVPKeyPointer::ParseKeyResult EVPKeyPointer::TryParsePublicKeyPEM(
+    const Buffer<const unsigned char>& buffer) {
+  auto bp = BIOPointer::New(buffer.data, buffer.len);
+  if (!bp)
+    return ParseKeyResult(PKParseError::FAILED);
+
+  // Try parsing as SubjectPublicKeyInfo (SPKI) first.
+  if (auto ret = TryParsePublicKeyInner(bp, "PUBLIC KEY",
+      [](const unsigned char** p, long l) {  // NOLINT(runtime/int)
+        return d2i_PUBKEY(nullptr, p, l);
+      })) {
+    return ret;
+  }
+
+  // Maybe it is PKCS#1.
+  if (auto ret = TryParsePublicKeyInner(bp, "RSA PUBLIC KEY",
+      [](const unsigned char** p, long l) {  // NOLINT(runtime/int)
+        return d2i_PublicKey(EVP_PKEY_RSA, nullptr, p, l);
+      })) {
+    return ret;
+  }
+
+  // X.509 fallback.
+  if (auto ret = TryParsePublicKeyInner(bp, "CERTIFICATE",
+      [](const unsigned char** p, long l) {  // NOLINT(runtime/int)
+        X509Pointer x509(d2i_X509(nullptr, p, l));
+        return x509 ? X509_get_pubkey(x509.get()) : nullptr;
+      })) {
+    return ret;
+  };
+
+  return ParseKeyResult(PKParseError::NOT_RECOGNIZED);
+}
+
+EVPKeyPointer::ParseKeyResult EVPKeyPointer::TryParsePublicKey(
+    PKFormatType format,
+    PKEncodingType encoding,
+    const Buffer<const unsigned char>& buffer) {
+  if (format == PKFormatType::PEM) {
+    return TryParsePublicKeyPEM(buffer);
+  }
+
+  if (format != PKFormatType::DER) {
+    return ParseKeyResult(PKParseError::FAILED);
+  }
+
+  const unsigned char* start = buffer.data;
+
+  EVP_PKEY* key = nullptr;
+
+  if (encoding == PKEncodingType::PKCS1 &&
+      (key = d2i_PublicKey(EVP_PKEY_RSA, nullptr, &start, buffer.len))) {
+    return EVPKeyPointer::ParseKeyResult(EVPKeyPointer(key));
+  }
+
+  if (encoding == PKEncodingType::SPKI &&
+      (key = d2i_PUBKEY(nullptr, &start, buffer.len))) {
+    return EVPKeyPointer::ParseKeyResult(EVPKeyPointer(key));
+  }
+
+  return ParseKeyResult(PKParseError::FAILED);
+}
+
+EVPKeyPointer::ParseKeyResult EVPKeyPointer::TryParsePrivateKey(
+    PKFormatType format,
+    PKEncodingType encoding,
+    std::optional<Buffer<char>> maybe_passphrase,
+    const Buffer<const unsigned char>& buffer) {
+
+  static auto keyOrError = [&](EVPKeyPointer pkey, bool had_passphrase = false) {
+    if (int err = ERR_peek_error()) {
+      if (ERR_GET_LIB(err) == ERR_LIB_PEM &&
+          ERR_GET_REASON(err) == PEM_R_BAD_PASSWORD_READ &&
+          !had_passphrase) {
+        return ParseKeyResult(PKParseError::NEED_PASSPHRASE);
+      }
+      return ParseKeyResult(PKParseError::FAILED, err);
+    }
+    if (!pkey) return ParseKeyResult(PKParseError::FAILED);
+    return ParseKeyResult(std::move(pkey));
+  };
+
+  Buffer<char>* passphrase = nullptr;
+  if (maybe_passphrase.has_value()) {
+    passphrase = &maybe_passphrase.value();
+  }
+
+  auto bio = BIOPointer::New(buffer);
+  if (!bio) return ParseKeyResult(PKParseError::FAILED);
+
+  if (format == PKFormatType::PEM) {
+    auto key = PEM_read_bio_PrivateKey(bio.get(), nullptr, PasswordCallback, passphrase);
+    return keyOrError(EVPKeyPointer(key), maybe_passphrase.has_value());
+  }
+
+  if (format != PKFormatType::DER) {
+    return ParseKeyResult(PKParseError::FAILED);
+  }
+
+  switch (encoding) {
+    case PKEncodingType::PKCS1: {
+      auto key = d2i_PrivateKey_bio(bio.get(), nullptr);
+      return keyOrError(EVPKeyPointer(key));
+    }
+    case PKEncodingType::PKCS8: {
+      if (IsEncryptedPrivateKeyInfo(buffer)) {
+        auto key = d2i_PKCS8PrivateKey_bio(bio.get(),
+                                           nullptr,
+                                           PasswordCallback,
+                                           passphrase);
+        return keyOrError(EVPKeyPointer(key), maybe_passphrase.has_value());
+      }
+
+      PKCS8Pointer p8inf(d2i_PKCS8_PRIV_KEY_INFO_bio(bio.get(), nullptr));
+      if (!p8inf) {
+        return ParseKeyResult(PKParseError::FAILED, ERR_peek_error());
+      }
+      return keyOrError(EVPKeyPointer(EVP_PKCS82PKEY(p8inf.get())));
+    }
+    case PKEncodingType::SEC1: {
+      auto key = d2i_PrivateKey_bio(bio.get(), nullptr);
+      return keyOrError(EVPKeyPointer(key));
+    }
+    default: {
+      return ParseKeyResult(PKParseError::FAILED, ERR_peek_error());
+    }
+  };
+}
+
 }  // namespace ncrypto
