@@ -1218,6 +1218,11 @@ class V8_EXPORT_PRIVATE SimplifiedOperatorBuilder final
       const FastApiCallFunctionVector& c_candidate_functions,
       FeedbackSource const& feedback, CallDescriptor* descriptor);
 
+#ifdef V8_ENABLE_CONTINUATION_PRESERVED_EMBEDDER_DATA
+  const Operator* GetContinuationPreservedEmbedderData();
+  const Operator* SetContinuationPreservedEmbedderData();
+#endif  // V8_ENABLE_CONTINUATION_PRESERVED_EMBEDDER_DATA
+
  private:
   Zone* zone() const { return zone_; }
 
@@ -1267,8 +1272,10 @@ class SimplifiedNodeWrapperBase : public NodeWrapper {
 
 class FastApiCallNode final : public SimplifiedNodeWrapperBase {
  public:
-  explicit constexpr FastApiCallNode(Node* node)
-      : SimplifiedNodeWrapperBase(node) {
+  explicit FastApiCallNode(Node* node)
+      : SimplifiedNodeWrapperBase(node),
+        c_arg_count_(FastCallArgumentCount(node)),
+        slow_arg_count_(SlowCallArgumentCount(node)) {
     DCHECK_EQ(IrOpcode::kFastApiCall, node->opcode());
   }
 
@@ -1280,31 +1287,65 @@ class FastApiCallNode final : public SimplifiedNodeWrapperBase {
   INPUTS(DEFINE_INPUT_ACCESSORS)
 #undef INPUTS
 
-  // Besides actual arguments, FastApiCall nodes also take:
-  static constexpr int kSlowTargetInputCount = 1;
-  static constexpr int kFastReceiverInputCount = 1;
-  static constexpr int kSlowReceiverInputCount = 1;
-  static constexpr int kExtraInputCount = kFastReceiverInputCount;
-
-  static constexpr int kArityInputCount = 1;
-  static constexpr int kNewTargetInputCount = 1;
-  static constexpr int kHolderInputCount = 1;
-  static constexpr int kContextAndFrameStateInputCount = 2;
-  static constexpr int kEffectAndControlInputCount = 2;
-  static constexpr int kSlowCallExtraInputCount =
-      kSlowTargetInputCount + kArityInputCount + kNewTargetInputCount +
-      kSlowReceiverInputCount + kHolderInputCount +
-      kContextAndFrameStateInputCount + kEffectAndControlInputCount;
-
-  static constexpr int kSlowCallDataArgumentIndex = 3;
-
-  // This is the arity fed into FastApiCallArguments.
-  static constexpr int ArityForArgc(int c_arg_count, int js_arg_count) {
-    return c_arg_count + js_arg_count + kEffectAndControlInputCount;
+  // Callback data passed to fast calls via FastApiCallbackOptions struct.
+  constexpr int CallbackDataIndex() const {
+    // The last fast argument is the callback data.
+    return FastCallArgumentCount() - 1;
+  }
+  TNode<Object> CallbackData() const {
+    return TNode<Object>::UncheckedCast(
+        NodeProperties::GetValueInput(node(), CallbackDataIndex()));
   }
 
-  int FastCallArgumentCount() const;
-  int SlowCallArgumentCount() const;
+  // Context passed to slow fallback.
+  constexpr int ContextIndex() const {
+    // The last slow call argument is the frame state, the one before is the
+    // context.
+    return SlowCallArgumentIndex(SlowCallArgumentCount() - kFrameState - 1);
+  }
+  TNode<Object> Context() const {
+    return TNode<Object>::UncheckedCast(
+        NodeProperties::GetValueInput(node(), ContextIndex()));
+  }
+
+  // Frame state to slow fallback.
+  constexpr int FrameStateIndex() const {
+    // The last slow call argument is the frame state.
+    return SlowCallArgumentIndex(SlowCallArgumentCount() - 1);
+  }
+
+  // Besides actual C arguments (which already include receiver), FastApiCall
+  // nodes also take extra arguments for fast call and a pack of arguments for
+  // generating a slow call.
+  // Extra fast arguments:
+  //  - callback data (passed to fast callback via FastApiCallbackOptions
+  //    struct),
+  static constexpr int kCallbackData = 1;
+
+  // A pack of arguments required for a call to slow version (one of the
+  // CallApiCallbackOptimizedXXX builtins) includes:
+  //  - builtin target code,
+  static constexpr int kSlowCodeTarget = 1;
+  //  - params for builtin including context plus JS arguments including
+  //    receiver, see CallApiCallbackOptimizedDescriptor. This value is
+  //    provided as |slow_arg_count|,
+  //  - a frame state.
+  static constexpr int kFrameState = 1;
+
+  // This is the number of inputs fed into FastApiCall operator.
+  // |slow_arg_count| is the number of params for the slow builtin plus
+  // JS arguments including receiver.
+  static constexpr int ArityForArgc(int c_arg_count, int slow_arg_count) {
+    return c_arg_count + kCallbackData + kSlowCodeTarget + slow_arg_count +
+           kFrameState;
+  }
+
+  constexpr int CArgumentCount() const { return c_arg_count_; }
+
+  constexpr int FastCallArgumentCount() const {
+    return CArgumentCount() + kCallbackData;
+  }
+  constexpr int SlowCallArgumentCount() const { return slow_arg_count_; }
 
   constexpr int FirstFastCallArgumentIndex() const {
     return ReceiverIndex() + 1;
@@ -1318,8 +1359,10 @@ class FastApiCallNode final : public SimplifiedNodeWrapperBase {
         NodeProperties::GetValueInput(node(), FastCallArgumentIndex(i)));
   }
 
-  int FirstSlowCallArgumentIndex() const { return FastCallArgumentCount(); }
-  int SlowCallArgumentIndex(int i) const {
+  constexpr int FirstSlowCallArgumentIndex() const {
+    return FastCallArgumentCount();
+  }
+  constexpr int SlowCallArgumentIndex(int i) const {
     return FirstSlowCallArgumentIndex() + i;
   }
   TNode<Object> SlowCallArgument(int i) const {
@@ -1327,6 +1370,13 @@ class FastApiCallNode final : public SimplifiedNodeWrapperBase {
     return TNode<Object>::UncheckedCast(
         NodeProperties::GetValueInput(node(), SlowCallArgumentIndex(i)));
   }
+
+ private:
+  static int FastCallArgumentCount(Node* node);
+  static int SlowCallArgumentCount(Node* node);
+
+  const int c_arg_count_;
+  const int slow_arg_count_;
 };
 
 #undef DEFINE_INPUT_ACCESSORS
