@@ -5,15 +5,18 @@
 #ifndef V8_OBJECTS_JS_FUNCTION_H_
 #define V8_OBJECTS_JS_FUNCTION_H_
 
-#include "src/base/optional.h"
+#include <optional>
+
 #include "src/objects/code-kind.h"
 #include "src/objects/js-objects.h"
 
 // Has to be the last include (doesn't have include guards):
 #include "src/objects/object-macros.h"
 
-namespace v8 {
-namespace internal {
+namespace v8::internal {
+
+class AbstractCode;
+class ClosureFeedbackCellArray;
 
 #include "torque-generated/src/objects/js-function-tq.inc"
 
@@ -43,9 +46,9 @@ class JSBoundFunction
           JSBoundFunction, JSFunctionOrBoundFunctionOrWrappedFunction> {
  public:
   static MaybeHandle<String> GetName(Isolate* isolate,
-                                     Handle<JSBoundFunction> function);
+                                     DirectHandle<JSBoundFunction> function);
   static Maybe<int> GetLength(Isolate* isolate,
-                              Handle<JSBoundFunction> function);
+                              DirectHandle<JSBoundFunction> function);
 
   // Dispatched behavior.
   DECL_PRINTER(JSBoundFunction)
@@ -53,7 +56,7 @@ class JSBoundFunction
 
   // The bound function's string representation implemented according
   // to ES6 section 19.2.3.5 Function.prototype.toString ( ).
-  static Handle<String> ToString(Handle<JSBoundFunction> function);
+  static Handle<String> ToString(DirectHandle<JSBoundFunction> function);
 
   TQ_OBJECT_CONSTRUCTORS(JSBoundFunction)
 };
@@ -64,13 +67,13 @@ class JSWrappedFunction
           JSWrappedFunction, JSFunctionOrBoundFunctionOrWrappedFunction> {
  public:
   static MaybeHandle<String> GetName(Isolate* isolate,
-                                     Handle<JSWrappedFunction> function);
+                                     DirectHandle<JSWrappedFunction> function);
   static Maybe<int> GetLength(Isolate* isolate,
-                              Handle<JSWrappedFunction> function);
+                              DirectHandle<JSWrappedFunction> function);
   // https://tc39.es/proposal-shadowrealm/#sec-wrappedfunctioncreate
-  static MaybeHandle<Object> Create(Isolate* isolate,
-                                    Handle<NativeContext> creation_context,
-                                    Handle<JSReceiver> value);
+  static MaybeHandle<Object> Create(
+      Isolate* isolate, DirectHandle<NativeContext> creation_context,
+      Handle<JSReceiver> value);
 
   // Dispatched behavior.
   DECL_PRINTER(JSWrappedFunction)
@@ -78,7 +81,7 @@ class JSWrappedFunction
 
   // The wrapped function's string representation implemented according
   // to ES6 section 19.2.3.5 Function.prototype.toString ( ).
-  static Handle<String> ToString(Handle<JSWrappedFunction> function);
+  static Handle<String> ToString(DirectHandle<JSWrappedFunction> function);
 
   TQ_OBJECT_CONSTRUCTORS(JSWrappedFunction)
 };
@@ -109,7 +112,8 @@ class JSFunction : public TorqueGeneratedJSFunction<
   inline Tagged<NativeContext> native_context();
   inline int length();
 
-  static Handle<String> GetName(Isolate* isolate, Handle<JSFunction> function);
+  static Handle<String> GetName(Isolate* isolate,
+                                DirectHandle<JSFunction> function);
 
   // [code]: The generated code object for this function.  Executed
   // when the function is invoked, e.g. foo() or new foo(). See
@@ -119,23 +123,30 @@ class JSFunction : public TorqueGeneratedJSFunction<
   // optimized code object, or when reading from the background thread.
   // Storing a builtin doesn't require release semantics because these objects
   // are fully initialized.
-  DECL_ACCESSORS(code, Tagged<Code>)
-  DECL_RELEASE_ACQUIRE_ACCESSORS(code, Tagged<Code>)
+  DECL_CODE_POINTER_ACCESSORS(code)
 
   // Returns the raw content of the Code field. When reading from a background
   // thread, the code field may still be uninitialized, in which case the field
   // contains Smi::zero().
-  inline Tagged<Object> raw_code() const;
-  inline Tagged<Object> raw_code(AcquireLoadTag) const;
+  inline Tagged<Object> raw_code(IsolateForSandbox isolate) const;
+  inline Tagged<Object> raw_code(IsolateForSandbox isolate,
+                                 AcquireLoadTag) const;
 
   // Returns the address of the function code's instruction start.
-  inline Address instruction_start() const;
+  inline Address instruction_start(IsolateForSandbox isolate) const;
 
   // Get the abstract code associated with the function, which will either be
   // a InstructionStream object or a BytecodeArray.
   template <typename IsolateT>
   inline Tagged<AbstractCode> abstract_code(IsolateT* isolate);
 
+#ifdef V8_ENABLE_LEAPTIERING
+  inline void initialize_dispatch_handle(IsolateForSandbox isolate,
+                                         uint16_t parameter_count);
+  inline void clear_dispatch_handle();
+  inline JSDispatchHandle dispatch_handle();
+#endif  // V8_ENABLE_LEAPTIERING
+
   // The predicates for querying code kinds related to this function have
   // specific terminology:
   //
@@ -153,35 +164,38 @@ class JSFunction : public TorqueGeneratedJSFunction<
   // been already deoptimized but its code() still needs to be unlinked, which
   // will happen on its next activation.
 
-  bool HasAvailableHigherTierCodeThan(CodeKind kind) const;
+  bool HasAvailableHigherTierCodeThan(IsolateForSandbox isolate,
+                                      CodeKind kind) const;
   // As above but only considers available code kinds passing the filter mask.
-  bool HasAvailableHigherTierCodeThanWithFilter(CodeKind kind,
+  bool HasAvailableHigherTierCodeThanWithFilter(IsolateForSandbox isolate,
+                                                CodeKind kind,
                                                 CodeKinds filter_mask) const;
 
   // True, iff any generated code kind is attached/available to this function.
-  V8_EXPORT_PRIVATE bool HasAttachedOptimizedCode() const;
-  bool HasAvailableOptimizedCode() const;
+  V8_EXPORT_PRIVATE bool HasAttachedOptimizedCode(
+      IsolateForSandbox isolate) const;
+  bool HasAvailableOptimizedCode(IsolateForSandbox isolate) const;
 
-  bool HasAttachedCodeKind(CodeKind kind) const;
-  bool HasAvailableCodeKind(CodeKind kind) const;
+  bool HasAttachedCodeKind(IsolateForSandbox isolate, CodeKind kind) const;
+  bool HasAvailableCodeKind(IsolateForSandbox isolate, CodeKind kind) const;
 
-  base::Optional<CodeKind> GetActiveTier() const;
-  V8_EXPORT_PRIVATE bool ActiveTierIsIgnition() const;
-  bool ActiveTierIsBaseline() const;
-  bool ActiveTierIsMaglev() const;
-  bool ActiveTierIsTurbofan() const;
+  std::optional<CodeKind> GetActiveTier(IsolateForSandbox isolate) const;
+  V8_EXPORT_PRIVATE bool ActiveTierIsIgnition(IsolateForSandbox isolate) const;
+  bool ActiveTierIsBaseline(IsolateForSandbox isolate) const;
+  bool ActiveTierIsMaglev(IsolateForSandbox isolate) const;
+  bool ActiveTierIsTurbofan(IsolateForSandbox isolate) const;
 
   // Similar to SharedFunctionInfo::CanDiscardCompiled. Returns true, if the
   // attached code can be recreated at a later point by replacing it with
   // CompileLazy.
-  bool CanDiscardCompiled() const;
+  bool CanDiscardCompiled(IsolateForSandbox isolate) const;
 
   // Tells whether function's code object checks its tiering state (some code
   // kinds, e.g. TURBOFAN, ignore the tiering state).
-  inline bool ChecksTieringState();
+  inline bool ChecksTieringState(IsolateForSandbox isolate);
 
   inline TieringState tiering_state() const;
-  inline void set_tiering_state(TieringState state);
+  inline void set_tiering_state(IsolateForSandbox isolate, TieringState state);
   inline void reset_tiering_state();
 
   // Mark this function for lazy recompilation. The function will be recompiled
@@ -189,13 +203,13 @@ class JSFunction : public TorqueGeneratedJSFunction<
   void MarkForOptimization(Isolate* isolate, CodeKind target_kind,
                            ConcurrencyMode mode);
 
-  inline TieringState osr_tiering_state();
-  inline void set_osr_tiering_state(TieringState marker);
+  inline bool osr_tiering_in_progress();
+  inline void set_osr_tiering_in_progress(bool osr_in_progress);
 
   // Sets the interrupt budget based on whether the function has a feedback
   // vector and any optimized code.
   void SetInterruptBudget(Isolate* isolate,
-                          base::Optional<CodeKind> override_active_tier = {});
+                          std::optional<CodeKind> override_active_tier = {});
 
   // If slack tracking is active, it computes instance size of the initial map
   // with minimum permissible object slack.  If it is not active, it simply
@@ -222,10 +236,10 @@ class JSFunction : public TorqueGeneratedJSFunction<
   DECL_GETTER(feedback_vector, Tagged<FeedbackVector>)
   DECL_GETTER(has_feedback_vector, bool)
   V8_EXPORT_PRIVATE static void EnsureFeedbackVector(
-      Isolate* isolate, Handle<JSFunction> function,
+      Isolate* isolate, DirectHandle<JSFunction> function,
       IsCompiledScope* compiled_scope);
   static void CreateAndAttachFeedbackVector(Isolate* isolate,
-                                            Handle<JSFunction> function,
+                                            DirectHandle<JSFunction> function,
                                             IsCompiledScope* compiled_scope);
 
   // Functions related to closure feedback cell array that holds feedback cells
@@ -235,13 +249,14 @@ class JSFunction : public TorqueGeneratedJSFunction<
   inline bool has_closure_feedback_cell_array() const;
   inline Tagged<ClosureFeedbackCellArray> closure_feedback_cell_array() const;
   static void EnsureClosureFeedbackCellArray(
-      Handle<JSFunction> function, bool reset_budget_for_feedback_allocation);
+      DirectHandle<JSFunction> function,
+      bool reset_budget_for_feedback_allocation);
 
   // Initializes the feedback cell of |function|. In lite mode, this would be
   // initialized to the closure feedback cell array that holds the feedback
   // cells for create closure calls from this function. In the regular mode,
   // this allocates feedback vector.
-  static void InitializeFeedbackCell(Handle<JSFunction> function,
+  static void InitializeFeedbackCell(DirectHandle<JSFunction> function,
                                      IsCompiledScope* compiled_scope,
                                      bool reset_budget_for_feedback_allocation);
 
@@ -250,16 +265,17 @@ class JSFunction : public TorqueGeneratedJSFunction<
   void ClearAllTypeFeedbackInfoForTesting();
 
   // Resets function to clear compiled data after bytecode has been flushed.
-  inline bool NeedsResetDueToFlushedBytecode();
+  inline bool NeedsResetDueToFlushedBytecode(IsolateForSandbox isolate);
   inline void ResetIfCodeFlushed(
-      base::Optional<
+      IsolateForSandbox isolate,
+      std::optional<
           std::function<void(Tagged<HeapObject> object, ObjectSlot slot,
                              Tagged<HeapObject> target)>>
-          gc_notify_updated_slot = base::nullopt);
+          gc_notify_updated_slot = std::nullopt);
 
   // Returns if the closure's code field has to be updated because it has
   // stale baseline code.
-  inline bool NeedsResetDueToFlushedBaselineCode();
+  inline bool NeedsResetDueToFlushedBaselineCode(IsolateForSandbox isolate);
 
   // Returns if baseline code is a candidate for flushing. This method is called
   // from concurrent marking so we should be careful when accessing data fields.
@@ -271,11 +287,11 @@ class JSFunction : public TorqueGeneratedJSFunction<
   // The initial map for an object created by this constructor.
   DECL_GETTER(initial_map, Tagged<Map>)
 
-  static void SetInitialMap(Isolate* isolate, Handle<JSFunction> function,
+  static void SetInitialMap(Isolate* isolate, DirectHandle<JSFunction> function,
                             Handle<Map> map, Handle<HeapObject> prototype);
-  static void SetInitialMap(Isolate* isolate, Handle<JSFunction> function,
+  static void SetInitialMap(Isolate* isolate, DirectHandle<JSFunction> function,
                             Handle<Map> map, Handle<HeapObject> prototype,
-                            Handle<JSFunction> constructor);
+                            DirectHandle<JSFunction> constructor);
 
   DECL_GETTER(has_initial_map, bool)
   V8_EXPORT_PRIVATE static void EnsureHasInitialMap(
@@ -308,10 +324,11 @@ class JSFunction : public TorqueGeneratedJSFunction<
   DECL_GETTER(instance_prototype, Tagged<HeapObject>)
   DECL_GETTER(has_prototype_property, bool)
   DECL_GETTER(PrototypeRequiresRuntimeLookup, bool)
-  static void SetPrototype(Handle<JSFunction> function, Handle<Object> value);
+  static void SetPrototype(DirectHandle<JSFunction> function,
+                           Handle<Object> value);
 
   // Returns if this function has been compiled to native code yet.
-  inline bool is_compiled() const;
+  inline bool is_compiled(IsolateForSandbox isolate) const;
 
   static int GetHeaderSize(bool function_has_prototype_slot) {
     return function_has_prototype_slot ? JSFunction::kSizeWithPrototype
@@ -344,7 +361,7 @@ class JSFunction : public TorqueGeneratedJSFunction<
   // function name's length exceeds String::kMaxLength.
   static V8_WARN_UNUSED_RESULT bool SetName(Handle<JSFunction> function,
                                             Handle<Name> name,
-                                            Handle<String> prefix);
+                                            DirectHandle<String> prefix);
 
   // The function's name if it is configured, otherwise shared function info
   // debug name.
@@ -352,7 +369,7 @@ class JSFunction : public TorqueGeneratedJSFunction<
 
   // The function's string representation implemented according to
   // ES6 section 19.2.3.5 Function.prototype.toString ( ).
-  static Handle<String> ToString(Handle<JSFunction> function);
+  static Handle<String> ToString(DirectHandle<JSFunction> function);
 
   class BodyDescriptor;
 
@@ -374,12 +391,12 @@ class JSFunction : public TorqueGeneratedJSFunction<
   // TODO(jgruber): Currently at most one code kind can be attached. Consider
   // adding a NOT_COMPILED kind and changing this function to simply return the
   // kind if this becomes more convenient in the future.
-  CodeKinds GetAttachedCodeKinds() const;
+  CodeKinds GetAttachedCodeKinds(IsolateForSandbox isolate) const;
 
   // As above, but also considers locations outside of this JSFunction. For
   // example the optimized code cache slot in the feedback vector, and the
   // shared function info.
-  CodeKinds GetAvailableCodeKinds() const;
+  CodeKinds GetAvailableCodeKinds(IsolateForSandbox isolate) const;
 
  public:
   static constexpr int kSizeWithoutPrototype = kPrototypeOrInitialMapOffset;
@@ -388,8 +405,7 @@ class JSFunction : public TorqueGeneratedJSFunction<
   TQ_OBJECT_CONSTRUCTORS(JSFunction)
 };
 
-}  // namespace internal
-}  // namespace v8
+}  // namespace v8::internal
 
 #include "src/objects/object-macros-undef.h"
 
