@@ -33,13 +33,13 @@ RUNTIME_FUNCTION(Runtime_ThrowUnsupportedSuperError) {
 RUNTIME_FUNCTION(Runtime_ThrowConstructorNonCallableError) {
   HandleScope scope(isolate);
   DCHECK_EQ(1, args.length());
-  Handle<JSFunction> constructor = args.at<JSFunction>(0);
+  DirectHandle<JSFunction> constructor = args.at<JSFunction>(0);
   Handle<String> name(constructor->shared()->Name(), isolate);
 
-  Handle<Context> context = handle(constructor->native_context(), isolate);
+  DirectHandle<Context> context(constructor->native_context(), isolate);
   DCHECK(IsNativeContext(*context));
   Handle<JSFunction> realm_type_error_function(
-      JSFunction::cast(context->get(Context::TYPE_ERROR_FUNCTION_INDEX)),
+      Cast<JSFunction>(context->get(Context::TYPE_ERROR_FUNCTION_INDEX)),
       isolate);
   if (name->length() == 0) {
     THROW_NEW_ERROR_RETURN_FAILURE(
@@ -75,12 +75,13 @@ RUNTIME_FUNCTION(Runtime_ThrowSuperNotCalled) {
 
 namespace {
 
-Object ThrowNotSuperConstructor(Isolate* isolate, Handle<Object> constructor,
-                                Handle<JSFunction> function) {
-  Handle<String> super_name;
+Tagged<Object> ThrowNotSuperConstructor(Isolate* isolate,
+                                        Handle<Object> constructor,
+                                        DirectHandle<JSFunction> function) {
+  DirectHandle<String> super_name;
   if (IsJSFunction(*constructor)) {
-    super_name = handle(Handle<JSFunction>::cast(constructor)->shared()->Name(),
-                        isolate);
+    super_name =
+        direct_handle(Cast<JSFunction>(constructor)->shared()->Name(), isolate);
   } else if (IsOddball(*constructor)) {
     DCHECK(IsNull(*constructor, isolate));
     super_name = isolate->factory()->null_string();
@@ -110,7 +111,7 @@ RUNTIME_FUNCTION(Runtime_ThrowNotSuperConstructor) {
   HandleScope scope(isolate);
   DCHECK_EQ(2, args.length());
   Handle<Object> constructor = args.at(0);
-  Handle<JSFunction> function = args.at<JSFunction>(1);
+  DirectHandle<JSFunction> function = args.at<JSFunction>(1);
   return ThrowNotSuperConstructor(isolate, constructor, function);
 }
 
@@ -121,7 +122,7 @@ Handle<Name> KeyToName(Isolate* isolate, Handle<Object> key) {
   static_assert((std::is_same<Dictionary, SwissNameDictionary>::value ||
                  std::is_same<Dictionary, NameDictionary>::value));
   DCHECK(IsName(*key));
-  return Handle<Name>::cast(key);
+  return Cast<Name>(key);
 }
 
 template <>
@@ -138,8 +139,9 @@ Handle<Name> KeyToName<NumberDictionary>(Isolate* isolate, Handle<Object> key) {
 //    shared name.
 template <typename Dictionary>
 MaybeHandle<Object> GetMethodAndSetName(Isolate* isolate,
-                                        RuntimeArguments& args, Smi index,
-                                        Handle<String> name_prefix,
+                                        RuntimeArguments& args,
+                                        Tagged<Smi> index,
+                                        DirectHandle<String> name_prefix,
                                         Handle<Object> key) {
   int int_index = index.value();
 
@@ -169,8 +171,8 @@ MaybeHandle<Object> GetMethodAndSetName(Isolate* isolate,
 // This is a simplified version of GetMethodAndSetName()
 // function above that is used when it's guaranteed that the method has
 // shared name.
-Object GetMethodWithSharedName(Isolate* isolate, RuntimeArguments& args,
-                               Object index) {
+Tagged<Object> GetMethodWithSharedName(Isolate* isolate, RuntimeArguments& args,
+                                       Tagged<Object> index) {
   DisallowGarbageCollection no_gc;
   int int_index = Smi::ToInt(index);
 
@@ -179,7 +181,7 @@ Object GetMethodWithSharedName(Isolate* isolate, RuntimeArguments& args,
     return args[int_index];
   }
 
-  Handle<JSFunction> method = args.at<JSFunction>(int_index);
+  DirectHandle<JSFunction> method = args.at<JSFunction>(int_index);
   DCHECK(method->shared()->HasSharedName());
   return *method;
 }
@@ -191,9 +193,9 @@ Handle<Dictionary> ShallowCopyDictionaryTemplate(
       Dictionary::ShallowCopy(isolate, dictionary_template);
   // Clone all AccessorPairs in the dictionary.
   for (InternalIndex i : dictionary->IterateEntries()) {
-    Object value = dictionary->ValueAt(i);
+    Tagged<Object> value = dictionary->ValueAt(i);
     if (IsAccessorPair(value)) {
-      Handle<AccessorPair> pair(AccessorPair::cast(value), isolate);
+      DirectHandle<AccessorPair> pair(Cast<AccessorPair>(value), isolate);
       pair = AccessorPair::Copy(isolate, pair);
       dictionary->ValueAtPut(i, *pair);
     }
@@ -207,18 +209,18 @@ bool SubstituteValues(Isolate* isolate, Handle<Dictionary> dictionary,
   // Replace all indices with proper methods.
   ReadOnlyRoots roots(isolate);
   for (InternalIndex i : dictionary->IterateEntries()) {
-    Object maybe_key = dictionary->KeyAt(i);
+    Tagged<Object> maybe_key = dictionary->KeyAt(i);
     if (!Dictionary::IsKey(roots, maybe_key)) continue;
     Handle<Object> key(maybe_key, isolate);
     Handle<Object> value(dictionary->ValueAt(i), isolate);
     if (IsAccessorPair(*value)) {
-      Handle<AccessorPair> pair = Handle<AccessorPair>::cast(value);
-      Object tmp = pair->getter();
+      auto pair = Cast<AccessorPair>(value);
+      Tagged<Object> tmp = pair->getter();
       if (IsSmi(tmp)) {
         Handle<Object> result;
         ASSIGN_RETURN_ON_EXCEPTION_VALUE(
             isolate, result,
-            GetMethodAndSetName<Dictionary>(isolate, args, Smi::cast(tmp),
+            GetMethodAndSetName<Dictionary>(isolate, args, Cast<Smi>(tmp),
                                             isolate->factory()->get_string(),
                                             key),
             false);
@@ -229,7 +231,7 @@ bool SubstituteValues(Isolate* isolate, Handle<Dictionary> dictionary,
         Handle<Object> result;
         ASSIGN_RETURN_ON_EXCEPTION_VALUE(
             isolate, result,
-            GetMethodAndSetName<Dictionary>(isolate, args, Smi::cast(tmp),
+            GetMethodAndSetName<Dictionary>(isolate, args, Cast<Smi>(tmp),
                                             isolate->factory()->set_string(),
                                             key),
             false);
@@ -239,7 +241,7 @@ bool SubstituteValues(Isolate* isolate, Handle<Dictionary> dictionary,
       Handle<Object> result;
       ASSIGN_RETURN_ON_EXCEPTION_VALUE(
           isolate, result,
-          GetMethodAndSetName<Dictionary>(isolate, args, Smi::cast(*value),
+          GetMethodAndSetName<Dictionary>(isolate, args, Cast<Smi>(*value),
                                           isolate->factory()->empty_string(),
                                           key),
           false);
@@ -251,33 +253,33 @@ bool SubstituteValues(Isolate* isolate, Handle<Dictionary> dictionary,
 
 template <typename Dictionary>
 void UpdateProtectors(Isolate* isolate, Handle<JSObject> receiver,
-                      Handle<Dictionary> properties_dictionary) {
+                      DirectHandle<Dictionary> properties_dictionary) {
   ReadOnlyRoots roots(isolate);
   for (InternalIndex i : properties_dictionary->IterateEntries()) {
-    Object maybe_key = properties_dictionary->KeyAt(i);
+    Tagged<Object> maybe_key = properties_dictionary->KeyAt(i);
     if (!Dictionary::IsKey(roots, maybe_key)) continue;
-    Handle<Name> name(Name::cast(maybe_key), isolate);
+    DirectHandle<Name> name(Cast<Name>(maybe_key), isolate);
     LookupIterator::UpdateProtector(isolate, receiver, name);
   }
 }
 
 void UpdateProtectors(Isolate* isolate, Handle<JSObject> receiver,
-                      Handle<DescriptorArray> properties_template) {
+                      DirectHandle<DescriptorArray> properties_template) {
   int nof_descriptors = properties_template->number_of_descriptors();
   for (InternalIndex i : InternalIndex::Range(nof_descriptors)) {
-    Handle<Name> name(properties_template->GetKey(i), isolate);
+    DirectHandle<Name> name(properties_template->GetKey(i), isolate);
     LookupIterator::UpdateProtector(isolate, receiver, name);
   }
 }
 
 bool AddDescriptorsByTemplate(
-    Isolate* isolate, Handle<Map> map,
-    Handle<DescriptorArray> descriptors_template,
+    Isolate* isolate, DirectHandle<Map> map,
+    DirectHandle<DescriptorArray> descriptors_template,
     Handle<NumberDictionary> elements_dictionary_template,
     Handle<JSObject> receiver, RuntimeArguments& args) {
   int nof_descriptors = descriptors_template->number_of_descriptors();
 
-  Handle<DescriptorArray> descriptors =
+  DirectHandle<DescriptorArray> descriptors =
       DescriptorArray::Allocate(isolate, nof_descriptors, 0);
 
   Handle<NumberDictionary> elements_dictionary =
@@ -297,21 +299,21 @@ bool AddDescriptorsByTemplate(
       count++;
     }
   }
-  Handle<PropertyArray> property_array =
+  DirectHandle<PropertyArray> property_array =
       isolate->factory()->NewPropertyArray(count);
 
   // Read values from |descriptors_template| and store possibly post-processed
   // values into "instantiated" |descriptors| array.
   int field_index = 0;
   for (InternalIndex i : InternalIndex::Range(nof_descriptors)) {
-    Object value = descriptors_template->GetStrongValue(i);
+    Tagged<Object> value = descriptors_template->GetStrongValue(i);
     if (IsAccessorPair(value)) {
-      Handle<AccessorPair> pair = AccessorPair::Copy(
-          isolate, handle(AccessorPair::cast(value), isolate));
+      DirectHandle<AccessorPair> pair = AccessorPair::Copy(
+          isolate, handle(Cast<AccessorPair>(value), isolate));
       value = *pair;
     }
     DisallowGarbageCollection no_gc;
-    Name name = descriptors_template->GetKey(i);
+    Tagged<Name> name = descriptors_template->GetKey(i);
     // TODO(v8:5799): consider adding a ClassBoilerplate flag
     // "has_interesting_properties".
     if (name->IsInteresting(isolate)) {
@@ -329,8 +331,8 @@ bool AddDescriptorsByTemplate(
       } else {
         DCHECK_EQ(PropertyKind::kAccessor, details.kind());
         if (IsAccessorPair(value)) {
-          AccessorPair pair = AccessorPair::cast(value);
-          Object tmp = pair->getter();
+          Tagged<AccessorPair> pair = Cast<AccessorPair>(value);
+          Tagged<Object> tmp = pair->getter();
           if (IsSmi(tmp)) {
             pair->set_getter(GetMethodWithSharedName(isolate, args, tmp));
           }
@@ -354,10 +356,9 @@ bool AddDescriptorsByTemplate(
 
       property_array->set(field_index, value);
       field_index++;
-      descriptors->Set(i, name, MaybeObject::FromObject(FieldType::Any()),
-                       details);
+      descriptors->Set(i, name, FieldType::Any(), details);
     } else {
-      descriptors->Set(i, name, MaybeObject::FromObject(value), details);
+      descriptors->Set(i, name, value, details);
     }
   }
 
@@ -398,10 +399,10 @@ Handle<T> ToHandle(MaybeHandle<T> h) {
 
 template <typename Dictionary>
 bool AddDescriptorsByTemplate(
-    Isolate* isolate, Handle<Map> map,
+    Isolate* isolate, DirectHandle<Map> map,
     Handle<Dictionary> properties_dictionary_template,
     Handle<NumberDictionary> elements_dictionary_template,
-    Handle<FixedArray> computed_properties, Handle<JSObject> receiver,
+    DirectHandle<FixedArray> computed_properties, Handle<JSObject> receiver,
     RuntimeArguments& args) {
   int computed_properties_length = computed_properties->length();
 
@@ -422,12 +423,12 @@ bool AddDescriptorsByTemplate(
 
     ValueKind value_kind = ComputedEntryFlags::ValueKindBits::decode(flags);
     int key_index = ComputedEntryFlags::KeyIndexBits::decode(flags);
-    Smi value = Smi::FromInt(key_index + 1);  // Value follows name.
+    Tagged<Smi> value = Smi::FromInt(key_index + 1);  // Value follows name.
 
     Handle<Object> key = args.at(key_index);
     DCHECK(IsName(*key));
     uint32_t element;
-    Handle<Name> name = Handle<Name>::cast(key);
+    Handle<Name> name = Cast<Name>(key);
     if (name->AsArrayIndex(&element)) {
       ClassBoilerplate::AddToElementsTemplate(
           isolate, elements_dictionary, element, key_index, value_kind, value);
@@ -450,7 +451,8 @@ bool AddDescriptorsByTemplate(
     return false;
   }
 
-  UpdateProtectors(isolate, receiver, properties_dictionary);
+  UpdateProtectors(isolate, receiver,
+                   DirectHandle<Dictionary>(properties_dictionary));
 
   if (elements_dictionary->NumberOfElements() > 0) {
     if (!SubstituteValues<NumberDictionary>(isolate, elements_dictionary,
@@ -476,34 +478,34 @@ Handle<JSObject> CreateClassPrototype(Isolate* isolate) {
 
   // TODO(ishell) Support caching of zero in-object properties map
   // by ObjectLiteralMapFromCache().
-  Handle<Map> map = Map::Create(isolate, 0);
+  DirectHandle<Map> map = Map::Create(isolate, 0);
   return isolate->factory()->NewJSObjectFromMap(map);
 }
 
 bool InitClassPrototype(Isolate* isolate,
-                        Handle<ClassBoilerplate> class_boilerplate,
+                        DirectHandle<ClassBoilerplate> class_boilerplate,
                         Handle<JSObject> prototype,
                         Handle<HeapObject> prototype_parent,
-                        Handle<JSFunction> constructor,
+                        DirectHandle<JSFunction> constructor,
                         RuntimeArguments& args) {
   Handle<Map> map(prototype->map(), isolate);
   map = Map::CopyDropDescriptors(isolate, map);
   map->set_is_prototype_map(true);
   Map::SetPrototype(isolate, map, prototype_parent);
+  isolate->UpdateProtectorsOnSetPrototype(prototype, prototype_parent);
   constructor->set_prototype_or_initial_map(*prototype, kReleaseStore);
   map->SetConstructor(*constructor);
-  Handle<FixedArray> computed_properties(
+  DirectHandle<FixedArray> computed_properties(
       class_boilerplate->instance_computed_properties(), isolate);
   Handle<NumberDictionary> elements_dictionary_template(
-      NumberDictionary::cast(class_boilerplate->instance_elements_template()),
+      Cast<NumberDictionary>(class_boilerplate->instance_elements_template()),
       isolate);
 
   Handle<Object> properties_template(
       class_boilerplate->instance_properties_template(), isolate);
 
   if (IsDescriptorArray(*properties_template)) {
-    Handle<DescriptorArray> descriptors_template =
-        Handle<DescriptorArray>::cast(properties_template);
+    auto descriptors_template = Cast<DescriptorArray>(properties_template);
 
     // The size of the prototype object is known at this point.
     // So we can create it now and then add the rest instance methods to the
@@ -517,24 +519,16 @@ bool InitClassPrototype(Isolate* isolate,
     map->set_may_have_interesting_properties(true);
     map->set_construction_counter(Map::kNoSlackTracking);
 
-    if constexpr (V8_ENABLE_SWISS_NAME_DICTIONARY_BOOL) {
-      Handle<SwissNameDictionary> properties_dictionary_template =
-          Handle<SwissNameDictionary>::cast(properties_template);
-      return AddDescriptorsByTemplate(
-          isolate, map, properties_dictionary_template,
-          elements_dictionary_template, computed_properties, prototype, args);
-    } else {
-      Handle<NameDictionary> properties_dictionary_template =
-          Handle<NameDictionary>::cast(properties_template);
-      return AddDescriptorsByTemplate(
-          isolate, map, properties_dictionary_template,
-          elements_dictionary_template, computed_properties, prototype, args);
-    }
+    auto properties_dictionary_template =
+        Cast<PropertyDictionary>(properties_template);
+    return AddDescriptorsByTemplate(
+        isolate, map, properties_dictionary_template,
+        elements_dictionary_template, computed_properties, prototype, args);
   }
 }
 
 bool InitClassConstructor(Isolate* isolate,
-                          Handle<ClassBoilerplate> class_boilerplate,
+                          DirectHandle<ClassBoilerplate> class_boilerplate,
                           Handle<HeapObject> constructor_parent,
                           Handle<JSFunction> constructor,
                           RuntimeArguments& args) {
@@ -551,17 +545,16 @@ bool InitClassConstructor(Isolate* isolate,
   }
 
   Handle<NumberDictionary> elements_dictionary_template(
-      NumberDictionary::cast(class_boilerplate->static_elements_template()),
+      Cast<NumberDictionary>(class_boilerplate->static_elements_template()),
       isolate);
-  Handle<FixedArray> computed_properties(
+  DirectHandle<FixedArray> computed_properties(
       class_boilerplate->static_computed_properties(), isolate);
 
   Handle<Object> properties_template(
       class_boilerplate->static_properties_template(), isolate);
 
   if (IsDescriptorArray(*properties_template)) {
-    Handle<DescriptorArray> descriptors_template =
-        Handle<DescriptorArray>::cast(properties_template);
+    auto descriptors_template = Cast<DescriptorArray>(properties_template);
 
     return AddDescriptorsByTemplate(isolate, map, descriptors_template,
                                     elements_dictionary_template, constructor,
@@ -575,15 +568,15 @@ bool InitClassConstructor(Isolate* isolate,
     map->set_construction_counter(Map::kNoSlackTracking);
 
     if constexpr (V8_ENABLE_SWISS_NAME_DICTIONARY_BOOL) {
-      Handle<SwissNameDictionary> properties_dictionary_template =
-          Handle<SwissNameDictionary>::cast(properties_template);
+      auto properties_dictionary_template =
+          Cast<SwissNameDictionary>(properties_template);
 
       return AddDescriptorsByTemplate(
           isolate, map, properties_dictionary_template,
           elements_dictionary_template, computed_properties, constructor, args);
     } else {
-      Handle<NameDictionary> properties_dictionary_template =
-          Handle<NameDictionary>::cast(properties_template);
+      auto properties_dictionary_template =
+          Cast<NameDictionary>(properties_template);
       return AddDescriptorsByTemplate(
           isolate, map, properties_dictionary_template,
           elements_dictionary_template, computed_properties, constructor, args);
@@ -591,11 +584,10 @@ bool InitClassConstructor(Isolate* isolate,
   }
 }
 
-MaybeHandle<Object> DefineClass(Isolate* isolate,
-                                Handle<ClassBoilerplate> class_boilerplate,
-                                Handle<Object> super_class,
-                                Handle<JSFunction> constructor,
-                                RuntimeArguments& args) {
+MaybeHandle<Object> DefineClass(
+    Isolate* isolate, DirectHandle<ClassBoilerplate> class_boilerplate,
+    Handle<Object> super_class, Handle<JSFunction> constructor,
+    RuntimeArguments& args) {
   Handle<Object> prototype_parent;
   Handle<HeapObject> constructor_parent;
 
@@ -607,28 +599,25 @@ MaybeHandle<Object> DefineClass(Isolate* isolate,
     } else if (IsConstructor(*super_class)) {
       DCHECK(!IsJSFunction(*super_class) ||
              !IsResumableFunction(
-                 Handle<JSFunction>::cast(super_class)->shared()->kind()));
+                 Cast<JSFunction>(super_class)->shared()->kind()));
       ASSIGN_RETURN_ON_EXCEPTION(
           isolate, prototype_parent,
           Runtime::GetObjectProperty(isolate, super_class,
-                                     isolate->factory()->prototype_string()),
-          Object);
+                                     isolate->factory()->prototype_string()));
       if (!IsNull(*prototype_parent, isolate) &&
           !IsJSReceiver(*prototype_parent)) {
         THROW_NEW_ERROR(
             isolate, NewTypeError(MessageTemplate::kPrototypeParentNotAnObject,
-                                  prototype_parent),
-            Object);
+                                  prototype_parent));
       }
       // Create new handle to avoid |constructor_parent| corruption because of
       // |super_class| handle value overwriting via storing to
       // args[ClassBoilerplate::kPrototypeArgumentIndex] below.
-      constructor_parent = handle(HeapObject::cast(*super_class), isolate);
+      constructor_parent = handle(Cast<HeapObject>(*super_class), isolate);
     } else {
       THROW_NEW_ERROR(isolate,
                       NewTypeError(MessageTemplate::kExtendsValueNotConstructor,
-                                   super_class),
-                      Object);
+                                   super_class));
     }
   }
 
@@ -644,9 +633,9 @@ MaybeHandle<Object> DefineClass(Isolate* isolate,
   if (!InitClassConstructor(isolate, class_boilerplate, constructor_parent,
                             constructor, args) ||
       !InitClassPrototype(isolate, class_boilerplate, prototype,
-                          Handle<HeapObject>::cast(prototype_parent),
-                          constructor, args)) {
-    DCHECK(isolate->has_pending_exception());
+                          Cast<HeapObject>(prototype_parent), constructor,
+                          args)) {
+    DCHECK(isolate->has_exception());
     return MaybeHandle<Object>();
   }
   if (v8_flags.log_maps) {
@@ -669,7 +658,8 @@ MaybeHandle<Object> DefineClass(Isolate* isolate,
 RUNTIME_FUNCTION(Runtime_DefineClass) {
   HandleScope scope(isolate);
   DCHECK_LE(ClassBoilerplate::kFirstDynamicArgumentIndex, args.length());
-  Handle<ClassBoilerplate> class_boilerplate = args.at<ClassBoilerplate>(0);
+  DirectHandle<ClassBoilerplate> class_boilerplate =
+      args.at<ClassBoilerplate>(0);
   Handle<JSFunction> constructor = args.at<JSFunction>(1);
   Handle<Object> super_class = args.at(2);
   DCHECK_EQ(class_boilerplate->arguments_count(), args.length());
@@ -688,8 +678,7 @@ MaybeHandle<JSReceiver> GetSuperHolder(Isolate* isolate,
                                        SuperMode mode, PropertyKey* key) {
   if (IsAccessCheckNeeded(*home_object) &&
       !isolate->MayAccess(isolate->native_context(), home_object)) {
-    RETURN_ON_EXCEPTION(isolate, isolate->ReportFailedAccessCheck(home_object),
-                        JSReceiver);
+    RETURN_ON_EXCEPTION(isolate, isolate->ReportFailedAccessCheck(home_object));
     UNREACHABLE();
   }
 
@@ -701,9 +690,9 @@ MaybeHandle<JSReceiver> GetSuperHolder(Isolate* isolate,
             ? MessageTemplate::kNonObjectPropertyLoadWithProperty
             : MessageTemplate::kNonObjectPropertyStoreWithProperty;
     Handle<Name> name = key->GetName(isolate);
-    THROW_NEW_ERROR(isolate, NewTypeError(message, proto, name), JSReceiver);
+    THROW_NEW_ERROR(isolate, NewTypeError(message, proto, name));
   }
-  return Handle<JSReceiver>::cast(proto);
+  return Cast<JSReceiver>(proto);
 }
 
 MaybeHandle<Object> LoadFromSuper(Isolate* isolate, Handle<Object> receiver,
@@ -712,10 +701,10 @@ MaybeHandle<Object> LoadFromSuper(Isolate* isolate, Handle<Object> receiver,
   Handle<JSReceiver> holder;
   ASSIGN_RETURN_ON_EXCEPTION(
       isolate, holder,
-      GetSuperHolder(isolate, home_object, SuperMode::kLoad, key), Object);
+      GetSuperHolder(isolate, home_object, SuperMode::kLoad, key));
   LookupIterator it(isolate, receiver, *key, holder);
   Handle<Object> result;
-  ASSIGN_RETURN_ON_EXCEPTION(isolate, result, Object::GetProperty(&it), Object);
+  ASSIGN_RETURN_ON_EXCEPTION(isolate, result, Object::GetProperty(&it));
   return result;
 }
 
@@ -761,7 +750,7 @@ MaybeHandle<Object> StoreToSuper(Isolate* isolate, Handle<JSObject> home_object,
   Handle<JSReceiver> holder;
   ASSIGN_RETURN_ON_EXCEPTION(
       isolate, holder,
-      GetSuperHolder(isolate, home_object, SuperMode::kStore, key), Object);
+      GetSuperHolder(isolate, home_object, SuperMode::kStore, key));
   LookupIterator it(isolate, receiver, *key, holder);
   MAYBE_RETURN(Object::SetSuperProperty(&it, value, store_origin),
                MaybeHandle<Object>());
