@@ -174,7 +174,7 @@ function ClientRequest(input, options, cb) {
 
   const protocol = options.protocol || defaultAgent.protocol;
   let expectedProtocol = defaultAgent.protocol;
-  if (this.agent && this.agent.protocol)
+  if (this.agent?.protocol)
     expectedProtocol = this.agent.protocol;
 
   if (options.path) {
@@ -190,7 +190,7 @@ function ClientRequest(input, options, cb) {
   }
 
   const defaultPort = options.defaultPort ||
-                    (this.agent && this.agent.defaultPort);
+                    (this.agent?.defaultPort);
 
   const optsWithoutSignal = { __proto__: null, ...options };
 
@@ -347,8 +347,8 @@ function ClientRequest(input, options, cb) {
       opts = { ...optsWithoutSignal };
       if (opts.socketPath) {
         opts.path = opts.socketPath;
-      } else if (opts.path) {
-        opts.path = undefined;
+      } else {
+        opts.path &&= undefined;
       }
     }
     if (typeof opts.createConnection === 'function') {
@@ -553,7 +553,7 @@ function socketOnData(d) {
     socket.destroy();
     req.socket._hadError = true;
     emitErrorEvent(req, ret);
-  } else if (parser.incoming && parser.incoming.upgrade) {
+  } else if (parser.incoming?.upgrade) {
     // Upgrade (if status code 101) or CONNECT
     const bytesParsed = ret;
     const res = parser.incoming;
@@ -591,7 +591,7 @@ function socketOnData(d) {
       // Requested Upgrade or used CONNECT method, but have no handler.
       socket.destroy();
     }
-  } else if (parser.incoming && parser.incoming.complete &&
+  } else if (parser.incoming?.complete &&
              // When the status code is informational (100, 102-199),
              // the server will send a final response after this client
              // sends a request body, so we must not free the parser.
@@ -838,7 +838,7 @@ function tickOnSocket(req, socket) {
 
   if (
     req.timeout !== undefined ||
-    (req.agent && req.agent.options && req.agent.options.timeout)
+    (req.agent?.options?.timeout)
   ) {
     listenSocketTimeout(req);
   }
