@@ -423,7 +423,7 @@ OutgoingMessage.prototype._send = function _send(data, encoding, callback, byteL
 OutgoingMessage.prototype._writeRaw = _writeRaw;
 function _writeRaw(data, encoding, callback, size) {
   const conn = this[kSocket];
-  if (conn && conn.destroyed) {
+  if (conn?.destroyed) {
     // The socket was destroyed. If we're still trying to write to it,
     // then we haven't gotten the 'close' event yet.
     return false;
@@ -789,7 +789,7 @@ OutgoingMessage.prototype.getHeader = function getHeader(name) {
     return;
 
   const entry = headers[name.toLowerCase()];
-  return entry && entry[1];
+  return entry?.[1];
 };
 
 
@@ -1073,7 +1073,7 @@ OutgoingMessage.prototype.addTrailers = function addTrailers(headers) {
 };
 
 function onFinish(outmsg) {
-  if (outmsg && outmsg.socket && outmsg.socket._hadError) return;
+  if (outmsg?.socket?._hadError) return;
   outmsg.emit('finish');
 }
 
@@ -1188,7 +1188,7 @@ OutgoingMessage.prototype._finish = function _finish() {
 OutgoingMessage.prototype._flush = function _flush() {
   const socket = this[kSocket];
 
-  if (socket && socket.writable) {
+  if (socket?.writable) {
     // There might be remaining data in this.output; write it out
     const ret = this._flushOutput(socket);
 
