@@ -29,6 +29,7 @@ const {
   ObjectSetPrototypeOf,
   ReflectApply,
   Symbol,
+  SymbolAsyncDispose,
   SymbolFor,
 } = primordials;
 
@@ -81,7 +82,6 @@ const {
 const {
   kEmptyObject,
   promisify,
-  SymbolAsyncDispose,
 } = require('internal/util');
 const {
   validateInteger,
@@ -357,8 +357,7 @@ function writeHead(statusCode, reason, obj) {
     this.statusMessage = reason;
   } else {
     // writeHead(statusCode[, headers])
-    if (!this.statusMessage)
-      this.statusMessage = STATUS_CODES[statusCode] || 'unknown';
+    this.statusMessage ||= STATUS_CODES[statusCode] || 'unknown';
     obj ??= reason;
   }
   this.statusCode = statusCode;
@@ -510,9 +509,7 @@ function storeHTTPOptions(options) {
 
 function setupConnectionsTracking() {
   // Start connection handling
-  if (!this[kConnections]) {
-    this[kConnections] = new ConnectionsList();
-  }
+  this[kConnections] ||= new ConnectionsList();
 
   if (this[kConnectionsCheckingInterval]) {
     clearInterval(this[kConnectionsCheckingInterval]);
@@ -738,7 +735,7 @@ function connectionListenerInternal(server, socket) {
   socket.setEncoding = socketSetEncoding;
 
   // We only consume the socket if it has never been consumed before.
-  if (socket._handle && socket._handle.isStreamBase &&
+  if (socket._handle?.isStreamBase &&
       !socket._handle._consumed) {
     parser._consumed = true;
     socket._handle._consumed = true;
@@ -783,7 +780,7 @@ function socketOnDrain(socket, state) {
 }
 
 function socketOnTimeout() {
-  const req = this.parser && this.parser.incoming;
+  const req = this.parser?.incoming;
   const reqTimeout = req && !req.complete && req.emit('timeout', this);
   const res = this._httpMessage;
   const resTimeout = res && res.emit('timeout', this);
@@ -918,13 +915,12 @@ function onParserExecuteCommon(server, socket, parser, state, ret, d) {
     prepareError(ret, parser, d);
     debug('parse error', ret);
     socketOnError.call(socket, ret);
-  } else if (parser.incoming && parser.incoming.upgrade) {
+  } else if (parser.incoming?.upgrade) {
     // Upgrade or CONNECT
     const req = parser.incoming;
     debug('SERVER upgrade or connect', req.method);
 
-    if (!d)
-      d = parser.getCurrentBuffer();
+    d ||= parser.getCurrentBuffer();
 
     socket.removeListener('data', state.onData);
     socket.removeListener('end', state.onEnd);
@@ -962,8 +958,8 @@ function onParserExecuteCommon(server, socket, parser, state, ret, d) {
 }
 
 function clearIncoming(req) {
-  req = req || this;
-  const parser = req.socket && req.socket.parser;
+  req ||= this;
+  const parser = req.socket?.parser;
   // Reset the .incoming property so that the request object can be gc'ed.
   if (parser && parser.incoming === req) {
     if (req.readableEnded) {
@@ -1180,7 +1176,7 @@ function onSocketResume() {
 }
 
 function onSocketPause() {
-  if (this._handle && this._handle.reading) {
+  if (this._handle?.reading) {
     this._handle.reading = false;
     this._handle.readStop();
   }
