@@ -47,18 +47,18 @@ const {
   StringPrototypeSplit,
   Symbol,
   SymbolAsyncIterator,
+  SymbolDispose,
   SymbolFor,
 } = primordials;
 const kRejection = SymbolFor('nodejs.rejection');
 
-const { SymbolDispose, kEmptyObject } = require('internal/util');
+const { kEmptyObject, spliceOne } = require('internal/util');
 
 const {
   inspect,
   identicalSequenceRange,
 } = require('internal/util/inspect');
 
-let spliceOne;
 let FixedQueue;
 let kFirstEventParam;
 let kResistStopPropagation;
@@ -67,7 +67,6 @@ const {
   AbortError,
   codes: {
     ERR_INVALID_ARG_TYPE,
-    ERR_INVALID_THIS,
     ERR_UNHANDLED_ERROR,
   },
   genericNodeError,
@@ -106,9 +105,9 @@ function lazyEventEmitterAsyncResource() {
       AsyncResource,
     } = require('async_hooks');
 
-    const kEventEmitter = Symbol('kEventEmitter');
-    const kAsyncResource = Symbol('kAsyncResource');
     class EventEmitterReferencingAsyncResource extends AsyncResource {
+      #eventEmitter;
+
       /**
        * @param {EventEmitter} ee
        * @param {string} [type]
@@ -119,21 +118,21 @@ function lazyEventEmitterAsyncResource() {
        */
       constructor(ee, type, options) {
         super(type, options);
-        this[kEventEmitter] = ee;
+        this.#eventEmitter = ee;
       }
 
       /**
        * @type {EventEmitter}
        */
       get eventEmitter() {
-        if (this[kEventEmitter] === undefined)
-          throw new ERR_INVALID_THIS('EventEmitterReferencingAsyncResource');
-        return this[kEventEmitter];
+        return this.#eventEmitter;
       }
     }
 
     EventEmitterAsyncResource =
       class EventEmitterAsyncResource extends EventEmitter {
+        #asyncResource;
+
         /**
          * @param {{
          *   name?: string,
@@ -154,8 +153,7 @@ function lazyEventEmitterAsyncResource() {
           }
           super(options);
 
-          this[kAsyncResource] =
-            new EventEmitterReferencingAsyncResource(this, name, options);
+          this.#asyncResource = new EventEmitterReferencingAsyncResource(this, name, options);
         }
 
         /**
@@ -164,9 +162,7 @@ function lazyEventEmitterAsyncResource() {
          * @returns {boolean}
          */
         emit(event, ...args) {
-          if (this[kAsyncResource] === undefined)
-            throw new ERR_INVALID_THIS('EventEmitterAsyncResource');
-          const { asyncResource } = this;
+          const asyncResource = this.#asyncResource;
           ArrayPrototypeUnshift(args, super.emit, this, event);
           return ReflectApply(asyncResource.runInAsyncScope, asyncResource,
                               args);
@@ -176,36 +172,28 @@ function lazyEventEmitterAsyncResource() {
          * @returns {void}
          */
         emitDestroy() {
-          if (this[kAsyncResource] === undefined)
-            throw new ERR_INVALID_THIS('EventEmitterAsyncResource');
-          this.asyncResource.emitDestroy();
+          this.#asyncResource.emitDestroy();
         }
 
         /**
          * @type {number}
          */
         get asyncId() {
-          if (this[kAsyncResource] === undefined)
-            throw new ERR_INVALID_THIS('EventEmitterAsyncResource');
-          return this.asyncResource.asyncId();
+          return this.#asyncResource.asyncId();
         }
 
         /**
          * @type {number}
          */
         get triggerAsyncId() {
-          if (this[kAsyncResource] === undefined)
-            throw new ERR_INVALID_THIS('EventEmitterAsyncResource');
-          return this.asyncResource.triggerAsyncId();
+          return this.#asyncResource.triggerAsyncId();
         }
 
         /**
          * @type {EventEmitterReferencingAsyncResource}
          */
         get asyncResource() {
-          if (this[kAsyncResource] === undefined)
-            throw new ERR_INVALID_THIS('EventEmitterAsyncResource');
-          return this[kAsyncResource];
+          return this.#asyncResource;
         }
       };
   }
@@ -350,7 +338,7 @@ EventEmitter.init = function(opts) {
     this[kShapeMode] = true;
   }
 
-  this._maxListeners = this._maxListeners || undefined;
+  this._maxListeners ||= undefined;
 
 
   if (opts?.captureRejections) {
@@ -470,7 +458,7 @@ EventEmitter.prototype.emit = function emit(type, ...args) {
   if (events !== undefined) {
     if (doError && events[kErrorMonitor] !== undefined)
       this.emit(kErrorMonitor, ...args);
-    doError = (doError && events.error === undefined);
+    doError &&= events.error === undefined;
   } else if (!doError)
     return false;
 
@@ -717,8 +705,6 @@ EventEmitter.prototype.removeListener =
         if (position === 0)
           list.shift();
         else {
-          if (spliceOne === undefined)
-            spliceOne = require('internal/util').spliceOne;
           spliceOne(list, position);
         }
 
