@@ -15,6 +15,7 @@ const {
   SafeArrayIterator,
   SafePromisePrototypeFinally,
   Symbol,
+  SymbolAsyncDispose,
   Uint8Array,
   uncurryThis,
 } = primordials;
@@ -84,6 +85,7 @@ const {
   validateEncoding,
   validateInteger,
   validateObject,
+  validateOneOf,
   validateString,
   kValidateObjectAllowNullable,
 } = require('internal/validators');
@@ -98,7 +100,6 @@ const {
   promisify,
   isWindows,
   isMacOS,
-  SymbolAsyncDispose,
 } = require('internal/util');
 const EventEmitter = require('events');
 const { StringDecoder } = require('string_decoder');
@@ -619,8 +620,8 @@ async function cp(src, dest, options) {
 async function copyFile(src, dest, mode) {
   return await PromisePrototypeThen(
     binding.copyFile(
-      getValidatedPath(src, 'src'),
-      getValidatedPath(dest, 'dest'),
+      getValidatedPath(src, 'src', { expectFile: true, syscall: 'cp' }),
+      getValidatedPath(dest, 'dest', { expectFile: true, syscall: 'cp' }),
       mode,
       kUsePromises,
     ),
@@ -632,7 +633,7 @@ async function copyFile(src, dest, mode) {
 // Note that unlike fs.open() which uses numeric file descriptors,
 // fsPromises.open() uses the fs.FileHandle class.
 async function open(path, flags, mode) {
-  path = getValidatedPath(path);
+  path = getValidatedPath(path, 'path', { expectFile: true, syscall: 'open' });
   const flagsNumber = stringToFlags(flags);
   mode = parseFileMode(mode, 'mode', 0o666);
   return new FileHandle(await PromisePrototypeThen(
@@ -914,7 +915,7 @@ async function readdirRecursive(originalPath, options) {
       const { 0: path, 1: readdir } = ArrayPrototypePop(queue);
       for (const ent of readdir) {
         const direntPath = pathModule.join(path, ent);
-        const stat = binding.internalModuleStat(direntPath);
+        const stat = binding.internalModuleStat(binding, direntPath);
         ArrayPrototypePush(
           result,
           pathModule.relative(originalPath, direntPath),
@@ -972,9 +973,9 @@ async function readlink(path, options) {
   );
 }
 
-async function symlink(target, path, type_) {
-  let type = (typeof type_ === 'string' ? type_ : null);
-  if (isWindows && type === null) {
+async function symlink(target, path, type) {
+  validateOneOf(type, 'type', ['dir', 'file', 'junction', null, undefined]);
+  if (isWindows && type == null) {
     try {
       const absoluteTarget = pathModule.resolve(`${path}`, '..', `${target}`);
       type = (await stat(absoluteTarget)).isDirectory() ? 'dir' : 'file';
@@ -1222,7 +1223,7 @@ function isCustomIterable(obj) {
 async function appendFile(path, data, options) {
   options = getOptions(options, { encoding: 'utf8', mode: 0o666, flag: 'a' });
   options = copyObject(options);
-  options.flag = options.flag || 'a';
+  options.flag ||= 'a';
   return writeFile(path, data, options);
 }
 
