@@ -31,7 +31,6 @@ const {
   UVException,
   codes: {
     ERR_FS_EISDIR,
-    ERR_FS_INVALID_SYMLINK_TYPE,
     ERR_INCOMPATIBLE_OPTION_PAIR,
     ERR_INVALID_ARG_TYPE,
     ERR_INVALID_ARG_VALUE,
@@ -148,10 +147,7 @@ const kMaxUserId = 2 ** 32 - 1;
 
 let fs;
 function lazyLoadFs() {
-  if (!fs) {
-    fs = require('fs');
-  }
-  return fs;
+  return fs ??= require('fs');
 }
 
 function assertEncoding(encoding) {
@@ -165,7 +161,6 @@ class Dirent {
   constructor(name, type, path) {
     this.name = name;
     this.parentPath = path;
-    this.path = path;
     this[kType] = type;
   }
 
@@ -214,6 +209,15 @@ for (const name of ReflectOwnKeys(Dirent.prototype)) {
   };
 }
 
+ObjectDefineProperty(Dirent.prototype, 'path', {
+  __proto__: null,
+  get: deprecate(function() {
+    return this.parentPath;
+  }, 'dirent.path is deprecated in favor of dirent.parentPath', 'DEP0178'),
+  configurable: true,
+  enumerable: false,
+});
+
 function copyObject(source) {
   const target = {};
   for (const key in source)
@@ -638,22 +642,16 @@ function stringToFlags(flags, name = 'flags') {
 }
 
 const stringToSymlinkType = hideStackFrames((type) => {
-  let flags = 0;
-  if (typeof type === 'string') {
-    switch (type) {
-      case 'dir':
-        flags |= UV_FS_SYMLINK_DIR;
-        break;
-      case 'junction':
-        flags |= UV_FS_SYMLINK_JUNCTION;
-        break;
-      case 'file':
-        break;
-      default:
-        throw new ERR_FS_INVALID_SYMLINK_TYPE(type);
-    }
+  switch (type) {
+    case undefined:
+    case null:
+    case 'file':
+      return 0;
+    case 'dir':
+      return UV_FS_SYMLINK_DIR;
+    case 'junction':
+      return UV_FS_SYMLINK_JUNCTION;
   }
-  return flags;
 });
 
 // converts Date or number to a fractional UNIX timestamp
@@ -708,13 +706,38 @@ const validateOffsetLengthWrite = hideStackFrames(
   },
 );
 
-const validatePath = hideStackFrames((path, propName = 'path') => {
+/**
+ * Validates the given path based on the expected type (file or directory).
+ * @param {string} path - The path to validate.
+ * @param {string} [propName='path'] - The name of the property being validated.
+ * @param {object} options - Additional options for validation.
+ * @param {boolean} [options.expectFile] - If true, expects the path to be a file.
+ * @param {string} [options.syscall] - The name of the syscall.
+ * @throws {TypeError} Throws if the path is not a string, Uint8Array, or URL without null bytes.
+ * @throws {Error} Throws if the path does not meet the expected type (file).
+ */
+const validatePath = hideStackFrames((path, propName = 'path', options) => {
   if (typeof path !== 'string' && !isUint8Array(path)) {
     throw new ERR_INVALID_ARG_TYPE.HideStackFramesError(propName, ['string', 'Buffer', 'URL'], path);
   }
 
   const pathIsString = typeof path === 'string';
   const pathIsUint8Array = isUint8Array(path);
+  if (options?.expectFile) {
+    const lastCharacter = path[path.length - 1];
+    if (
+      lastCharacter === '/' || lastCharacter === 47 ||
+      (isWindows && (lastCharacter === '\\' || lastCharacter === 92))
+    ) {
+      throw new ERR_FS_EISDIR({
+        code: 'ERR_FS_EISDIR',
+        message: 'is a directory',
+        path,
+        syscall: options.syscall,
+        errno: ERR_FS_EISDIR,
+      });
+    }
+  }
 
   // We can only perform meaningful checks on strings and Uint8Arrays.
   if ((!pathIsString && !pathIsUint8Array) ||
@@ -730,11 +753,21 @@ const validatePath = hideStackFrames((path, propName = 'path') => {
   );
 });
 
-const getValidatedPath = hideStackFrames((fileURLOrPath, propName = 'path') => {
-  const path = toPathIfFileURL(fileURLOrPath);
-  validatePath(path, propName);
-  return path;
-});
+/**
+ * Validates and returns the given file URL or path based on the expected type (file or directory).
+ * @param {string|URL} fileURLOrPath - The file URL or path to validate.
+ * @param {string} [propName='path'] - The name of the property being validated.
+ * @param {object} options - Additional options for validation.
+ * @param {boolean} [options.expectFile] - If true, expects the path to be a file.
+ * @param {string} [options.syscall] - The name of the syscall.
+ * @returns {string} The validated path.
+ */
+const getValidatedPath =
+  hideStackFrames((fileURLOrPath, propName = 'path', options) => {
+    const path = toPathIfFileURL(fileURLOrPath);
+    validatePath(path, propName, options);
+    return path;
+  });
 
 const getValidatedFd = hideStackFrames((fd, propName = 'fd') => {
   if (ObjectIs(fd, -0)) {
