@@ -2,7 +2,9 @@
 
 const {
   ArrayPrototypeMap,
+  ArrayPrototypePush,
   Boolean,
+  FunctionPrototypeCall,
   JSONParse,
   ObjectKeys,
   ObjectPrototypeHasOwnProperty,
@@ -17,16 +19,6 @@ const {
   globalThis: { WebAssembly },
 } = primordials;
 
-/** @type {import('internal/util/types')} */
-let _TYPES = null;
-/**
- * Lazily loads and returns the internal/util/types module.
- */
-function lazyTypes() {
-  if (_TYPES !== null) { return _TYPES; }
-  return _TYPES = require('internal/util/types');
-}
-
 const {
   compileFunctionForCJSLoader,
 } = internalBinding('contextify');
@@ -36,7 +28,10 @@ const assert = require('internal/assert');
 const { readFileSync } = require('fs');
 const { dirname, extname, isAbsolute } = require('path');
 const {
+  assertBufferSource,
   loadBuiltinModule,
+  stringify,
+  stripTypeScriptTypes,
   stripBOM,
   urlToFilename,
 } = require('internal/modules/helpers');
@@ -55,7 +50,6 @@ let debug = require('internal/util/debuglog').debuglog('esm', (fn) => {
 const { emitExperimentalWarning, kEmptyObject, setOwnProperty, isWindows } = require('internal/util');
 const {
   ERR_UNKNOWN_BUILTIN_MODULE,
-  ERR_INVALID_RETURN_PROPERTY_VALUE,
 } = require('internal/errors').codes;
 const { maybeCacheSourceMap } = require('internal/source_map/source_map_cache');
 const moduleWrap = internalBinding('module_wrap');
@@ -75,28 +69,11 @@ function getSource(url) {
 /** @type {import('deps/cjs-module-lexer/lexer.js').parse} */
 let cjsParse;
 /**
- * Initializes the CommonJS module lexer parser.
- * If WebAssembly is available, it uses the optimized version from the dist folder.
- * Otherwise, it falls back to the JavaScript version from the lexer folder.
+ * Initializes the CommonJS module lexer parser using the JavaScript version.
+ * TODO(joyeecheung): Use `require('internal/deps/cjs-module-lexer/dist/lexer').initSync()`
+ * when cjs-module-lexer 1.4.0 is rolled in.
  */
-async function initCJSParse() {
-  if (typeof WebAssembly === 'undefined') {
-    initCJSParseSync();
-  } else {
-    const { parse, init } =
-        require('internal/deps/cjs-module-lexer/dist/lexer');
-    try {
-      await init();
-      cjsParse = parse;
-    } catch {
-      initCJSParseSync();
-    }
-  }
-}
-
 function initCJSParseSync() {
-  // TODO(joyeecheung): implement a binding that directly compiles using
-  // v8::WasmModuleObject::Compile() synchronously.
   if (cjsParse === undefined) {
     cjsParse = require('internal/deps/cjs-module-lexer/lexer').parse;
   }
@@ -105,44 +82,6 @@ function initCJSParseSync() {
 const translators = new SafeMap();
 exports.translators = translators;
 
-let DECODER = null;
-/**
- * Asserts that the given body is a buffer source (either a string, array buffer, or typed array).
- * Throws an error if the body is not a buffer source.
- * @param {string | ArrayBufferView | ArrayBuffer} body - The body to check.
- * @param {boolean} allowString - Whether or not to allow a string as a valid buffer source.
- * @param {string} hookName - The name of the hook being called.
- * @throws {ERR_INVALID_RETURN_PROPERTY_VALUE} If the body is not a buffer source.
- */
-function assertBufferSource(body, allowString, hookName) {
-  if (allowString && typeof body === 'string') {
-    return;
-  }
-  const { isArrayBufferView, isAnyArrayBuffer } = lazyTypes();
-  if (isArrayBufferView(body) || isAnyArrayBuffer(body)) {
-    return;
-  }
-  throw new ERR_INVALID_RETURN_PROPERTY_VALUE(
-    `${allowString ? 'string, ' : ''}array buffer, or typed array`,
-    hookName,
-    'source',
-    body,
-  );
-}
-
-/**
- * Converts a buffer or buffer-like object to a string.
- * @param {string | ArrayBuffer | ArrayBufferView} body - The buffer or buffer-like object to convert to a string.
- * @returns {string} The resulting string.
- */
-function stringify(body) {
-  if (typeof body === 'string') { return body; }
-  assertBufferSource(body, false, 'load');
-  const { TextDecoder } = require('internal/encoding');
-  DECODER = DECODER === null ? new TextDecoder() : DECODER;
-  return DECODER.decode(body);
-}
-
 /**
  * Converts a URL to a file path if the URL protocol is 'file:'.
  * @param {string} url - The URL to convert.
@@ -204,7 +143,7 @@ function loadCJSModule(module, source, url, filename, isMain) {
       }
       specifier = `${pathToFileURL(path)}`;
     }
-    const job = cascadedLoader.getModuleJobSync(specifier, url, importAttributes);
+    const job = cascadedLoader.getModuleJobForRequireInImportedCJS(specifier, url, importAttributes);
     job.runSync();
     return cjsCache.get(job.url).exports;
   };
@@ -244,14 +183,17 @@ function createCJSModuleWrap(url, source, isMain, loadCJS = loadCJSModule) {
 
   const { exportNames, module } = cjsPreparseModuleExports(filename, source);
   cjsCache.set(url, module);
-  const namesWithDefault = exportNames.has('default') ?
-    [...exportNames] : ['default', ...exportNames];
+
+  const wrapperNames = [...exportNames, 'module.exports'];
+  if (!exportNames.has('default')) {
+    ArrayPrototypePush(wrapperNames, 'default');
+  }
 
   if (isMain) {
     setOwnProperty(process, 'mainModule', module);
   }
 
-  return new ModuleWrap(url, undefined, namesWithDefault, function() {
+  return new ModuleWrap(url, undefined, wrapperNames, function() {
     debug(`Loading CJSModule ${url}`);
 
     if (!module.loaded) {
@@ -266,8 +208,7 @@ function createCJSModuleWrap(url, source, isMain, loadCJS = loadCJSModule) {
       ({ exports } = module);
     }
     for (const exportName of exportNames) {
-      if (!ObjectPrototypeHasOwnProperty(exports, exportName) ||
-          exportName === 'default') {
+      if (!ObjectPrototypeHasOwnProperty(exports, exportName) || exportName === 'default') {
         continue;
       }
       // We might trigger a getter -> dont fail.
@@ -280,16 +221,15 @@ function createCJSModuleWrap(url, source, isMain, loadCJS = loadCJSModule) {
       this.setExport(exportName, value);
     }
     this.setExport('default', exports);
+    this.setExport('module.exports', exports);
   }, module);
 }
 
 translators.set('commonjs-sync', function requireCommonJS(url, source, isMain) {
   initCJSParseSync();
-  assert(!isMain);  // This is only used by imported CJS modules.
 
   return createCJSModuleWrap(url, source, isMain, (module, source, url, filename, isMain) => {
     assert(module === CJSModule._cache[filename]);
-    assert(!isMain);
     wrapModuleLoad(filename, null, isMain);
   });
 });
@@ -297,17 +237,26 @@ translators.set('commonjs-sync', function requireCommonJS(url, source, isMain) {
 // Handle CommonJS modules referenced by `require` calls.
 // This translator function must be sync, as `require` is sync.
 translators.set('require-commonjs', (url, source, isMain) => {
+  initCJSParseSync();
   assert(cjsParse);
 
   return createCJSModuleWrap(url, source);
 });
 
+// Handle CommonJS modules referenced by `require` calls.
+// This translator function must be sync, as `require` is sync.
+translators.set('require-commonjs-typescript', (url, source, isMain) => {
+  emitExperimentalWarning('Type Stripping');
+  assert(cjsParse);
+  const code = stripTypeScriptTypes(stringify(source), url);
+  return createCJSModuleWrap(url, code);
+});
+
 // Handle CommonJS modules referenced by `import` statements or expressions,
 // or as the initial entry point when the ESM loader handles a CommonJS entry.
-translators.set('commonjs', async function commonjsStrategy(url, source,
-                                                            isMain) {
+translators.set('commonjs', function commonjsStrategy(url, source, isMain) {
   if (!cjsParse) {
-    await initCJSParse();
+    initCJSParseSync();
   }
 
   // For backward-compatibility, it's possible to return a nullish value for
@@ -325,7 +274,6 @@ translators.set('commonjs', async function commonjsStrategy(url, source,
     // Continue regardless of error.
   }
   return createCJSModuleWrap(url, source, isMain, cjsLoader);
-
 });
 
 /**
@@ -417,7 +365,6 @@ translators.set('builtin', function builtinStrategy(url) {
 
 // Strategy for loading a JSON file
 translators.set('json', function jsonStrategy(url, source) {
-  emitExperimentalWarning('Importing JSON modules');
   assertBufferSource(source, true, 'load');
   debug(`Loading JSONModule ${url}`);
   const pathname = StringPrototypeStartsWith(url, 'file:') ?
@@ -431,7 +378,7 @@ translators.set('json', function jsonStrategy(url, source) {
     modulePath = isWindows ?
       StringPrototypeReplaceAll(pathname, '/', '\\') : pathname;
     module = CJSModule._cache[modulePath];
-    if (module && module.loaded) {
+    if (module?.loaded) {
       const exports = module.exports;
       return new ModuleWrap(url, undefined, ['default'], function() {
         this.setExport('default', exports);
@@ -445,7 +392,7 @@ translators.set('json', function jsonStrategy(url, source) {
     // export, we have to check again if the module already exists or not.
     // TODO: remove CJS loader from here as well.
     module = CJSModule._cache[modulePath];
-    if (module && module.loaded) {
+    if (module?.loaded) {
       const exports = module.exports;
       return new ModuleWrap(url, undefined, ['default'], function() {
         this.setExport('default', exports);
@@ -486,8 +433,9 @@ translators.set('wasm', async function(url, source) {
 
   let compiled;
   try {
-    // TODO(joyeecheung): implement a binding that directly compiles using
-    // v8::WasmModuleObject::Compile() synchronously.
+    // TODO(joyeecheung): implement a translator that just uses
+    // compiled = new WebAssembly.Module(source) to compile it
+    // synchronously.
     compiled = await WebAssembly.compile(source);
   } catch (err) {
     err.message = errPath(url) + ': ' + err.message;
@@ -510,3 +458,21 @@ translators.set('wasm', async function(url, source) {
     }
   }).module;
 });
+
+// Strategy for loading a commonjs TypeScript module
+translators.set('commonjs-typescript', function(url, source) {
+  emitExperimentalWarning('Type Stripping');
+  assertBufferSource(source, true, 'load');
+  const code = stripTypeScriptTypes(stringify(source), url);
+  debug(`Translating TypeScript ${url}`);
+  return FunctionPrototypeCall(translators.get('commonjs'), this, url, code, false);
+});
+
+// Strategy for loading an esm TypeScript module
+translators.set('module-typescript', function(url, source) {
+  emitExperimentalWarning('Type Stripping');
+  assertBufferSource(source, true, 'load');
+  const code = stripTypeScriptTypes(stringify(source), url);
+  debug(`Translating TypeScript ${url}`);
+  return FunctionPrototypeCall(translators.get('module'), this, url, code, false);
+});
