@@ -214,7 +214,7 @@ function eos(stream, options, callback) {
     !readable &&
     (!willEmitClose || isReadable(stream)) &&
     (writableFinished || isWritable(stream) === false) &&
-    (wState == null || wState.pendingcb === 0)
+    (wState == null || wState.pendingcb === undefined || wState.pendingcb === 0)
   ) {
     process.nextTick(onclosed);
   } else if (
