@@ -30,6 +30,12 @@ const { tmpdir } = require('os');
 const { join, resolve, relative, matchesGlob } = require('path');
 const { fileURLToPath } = require('internal/url');
 const { kMappings, SourceMap } = require('internal/source_map/source_map');
+const {
+  codes: {
+    ERR_SOURCE_MAP_CORRUPT,
+    ERR_SOURCE_MAP_MISSING_SOURCE,
+  },
+} = require('internal/errors');
 const kCoverageFileRegex = /^coverage-(\d+)-(\d{13})-(\d+)\.json$/;
 const kIgnoreRegex = /\/\* node:coverage ignore next (?<count>\d+ )?\*\//;
 const kLineEndingRegex = /\r?\n$/u;
@@ -51,12 +57,19 @@ class CoverageLine {
 }
 
 class TestCoverage {
-  constructor(coverageDirectory, originalCoverageDirectory, workingDirectory, excludeGlobs, includeGlobs, thresholds) {
+  constructor(coverageDirectory,
+              originalCoverageDirectory,
+              workingDirectory,
+              excludeGlobs,
+              includeGlobs,
+              sourceMaps,
+              thresholds) {
     this.coverageDirectory = coverageDirectory;
     this.originalCoverageDirectory = originalCoverageDirectory;
     this.workingDirectory = workingDirectory;
     this.excludeGlobs = excludeGlobs;
     this.includeGlobs = includeGlobs;
+    this.sourceMaps = sourceMaps;
     this.thresholds = thresholds;
   }
 
@@ -335,7 +348,7 @@ class TestCoverage {
   mapCoverageWithSourceMap(coverage) {
     const { result } = coverage;
     const sourceMapCache = coverage['source-map-cache'];
-    if (!sourceMapCache) {
+    if (!this.sourceMaps || !sourceMapCache) {
       return result;
     }
     const newResult = new SafeMap();
@@ -348,6 +361,7 @@ class TestCoverage {
         continue;
       }
       const { data, lineLengths } = sourceMapCache[url];
+      if (!data) throw new ERR_SOURCE_MAP_CORRUPT(url);
       let offset = 0;
       const executedLines = ArrayPrototypeMap(lineLengths, (length, i) => {
         const coverageLine = new CoverageLine(i + 1, offset, null, length + 1);
@@ -390,6 +404,9 @@ class TestCoverage {
 
           newUrl ??= startEntry?.originalSource;
           const mappedLines = this.getLines(newUrl);
+          if (!mappedLines) {
+            throw new ERR_SOURCE_MAP_MISSING_SOURCE(newUrl, url);
+          }
           const mappedStartOffset = this.entryToOffset(startEntry, mappedLines);
           const mappedEndOffset = this.entryToOffset(endEntry, mappedLines) + 1;
           for (let l = startEntry.originalLine; l <= endEntry.originalLine; l++) {
@@ -479,16 +496,13 @@ function sortCoverageFiles(a, b) {
 
 function setupCoverage(options) {
   let originalCoverageDirectory = process.env.NODE_V8_COVERAGE;
-  const cwd = process.cwd();
 
-  if (originalCoverageDirectory) {
-    // NODE_V8_COVERAGE was already specified. Convert it to an absolute path
-    // and store it for later. The test runner will use a temporary directory
-    // so that no preexisting coverage files interfere with the results of the
-    // coverage report. Then, once the coverage is computed, move the coverage
-    // files back to the original NODE_V8_COVERAGE directory.
-    originalCoverageDirectory = resolve(cwd, originalCoverageDirectory);
-  }
+  // If NODE_V8_COVERAGE was already specified, convert it to an absolute path
+  // and store it for later. The test runner will use a temporary directory
+  // so that no preexisting coverage files interfere with the results of the
+  // coverage report. Then, once the coverage is computed, move the coverage
+  // files back to the original NODE_V8_COVERAGE directory.
+  originalCoverageDirectory &&= resolve(options.cwd, originalCoverageDirectory);
 
   const coverageDirectory = mkdtempSync(join(tmpdir(), 'node-coverage-'));
   const enabled = setupCoverageHooks(coverageDirectory);
@@ -504,9 +518,10 @@ function setupCoverage(options) {
   return new TestCoverage(
     coverageDirectory,
     originalCoverageDirectory,
-    cwd,
+    options.cwd,
     options.coverageExcludeGlobs,
     options.coverageIncludeGlobs,
+    options.sourceMaps,
     {
       __proto__: null,
       line: options.lineCoverage,
