@@ -34,7 +34,13 @@ const {
 } = require('internal/errors');
 const esmLoader = require('internal/modules/esm/loader');
 const { getOptionValue } = require('internal/options');
-const { fileURLToPath, toPathIfFileURL, URL } = require('internal/url');
+const {
+  fileURLToPath,
+  isURL,
+  pathToFileURL,
+  toPathIfFileURL,
+  URL,
+} = require('internal/url');
 const {
   emitExperimentalWarning,
   getStructuredStack,
@@ -49,13 +55,12 @@ const {
   validateInteger,
   validateObject,
   validateOneOf,
-  validateString,
 } = require('internal/validators');
 const { MockTimers } = require('internal/test_runner/mock/mock_timers');
 const { strictEqual, notStrictEqual } = require('assert');
-const { isBuiltin, Module, register } = require('module');
+const { Module } = require('internal/modules/cjs/loader');
 const { MessageChannel } = require('worker_threads');
-const { _load, _nodeModulePaths, _resolveFilename } = Module;
+const { _load, _nodeModulePaths, _resolveFilename, isBuiltin } = Module;
 function kDefaultFunction() {}
 const enableModuleMocking = getOptionValue('--experimental-test-module-mocks');
 const kMockSearchParam = 'node-test-mock';
@@ -65,7 +70,7 @@ const kMockUnknownMessage = 3;
 const kWaitTimeout = 5_000;
 const kBadExportsMessage = 'Cannot create mock because named exports ' +
   'cannot be applied to the provided default export.';
-const kSupportedFormats = ['builtin', 'commonjs', 'module'];
+const kSupportedFormats = ['builtin', 'commonjs', 'module', 'module-typescript', 'commonjs-typescript'];
 let sharedModuleState;
 
 class MockFunctionContext {
@@ -488,7 +493,11 @@ class MockTracker {
 
   module(specifier, options = kEmptyObject) {
     emitExperimentalWarning('Module mocking');
-    validateString(specifier, 'specifier');
+    if (typeof specifier !== 'string') {
+      if (!isURL(specifier))
+        throw new ERR_INVALID_ARG_TYPE('specifier', ['string', 'URL'], specifier);
+      specifier = `${specifier}`;
+    }
     validateObject(options, 'options');
     debug('module mock entry, specifier = "%s", options = %o', specifier, options);
 
@@ -508,7 +517,10 @@ class MockTracker {
 
     // Get the file that called this function. We need four stack frames:
     // vm context -> getStructuredStack() -> this function -> actual caller.
-    const caller = getStructuredStack()[3]?.getFileName();
+    const filename = getStructuredStack()[3]?.getFileName();
+    // If the caller is already a file URL, use it as is. Otherwise, convert it.
+    const hasFileProtocol = StringPrototypeStartsWith(filename, 'file://');
+    const caller = hasFileProtocol ? filename : pathToFileURL(filename).href;
     const { format, url } = sharedState.moduleLoader.resolveSync(
       mockSpecifier, caller, null,
     );
@@ -650,19 +662,22 @@ function setupSharedModuleState() {
     const { mock } = require('test');
     const mockExports = new SafeMap();
     const { port1, port2 } = new MessageChannel();
+    const moduleLoader = esmLoader.getOrInitializeCascadedLoader();
 
-    register('node:test/mock_loader', {
-      __proto__: null,
-      data: { __proto__: null, port: port2 },
-      transferList: [port2],
-    });
+    moduleLoader.register(
+      'internal/test_runner/mock/loader',
+      'node:',
+      { __proto__: null, port: port2 },
+      [port2],
+      true,
+    );
 
     sharedModuleState = {
       __proto__: null,
       loaderPort: port1,
       mockExports,
       mockMap: new SafeMap(),
-      moduleLoader: esmLoader.getOrInitializeCascadedLoader(),
+      moduleLoader,
     };
     mock._mockExports = mockExports;
     Module._load = FunctionPrototypeBind(cjsMockModuleLoad, sharedModuleState);
