@@ -10,6 +10,7 @@ const {
   ArrayPrototypeJoin,
   ArrayPrototypeMap,
   ArrayPrototypePush,
+  ArrayPrototypePushApply,
   ArrayPrototypeShift,
   ArrayPrototypeSlice,
   ArrayPrototypeSome,
@@ -17,6 +18,7 @@ const {
   ObjectAssign,
   PromisePrototypeThen,
   PromiseResolve,
+  PromiseWithResolvers,
   SafeMap,
   SafePromiseAll,
   SafePromiseAllReturnVoid,
@@ -32,7 +34,7 @@ const {
 
 const { spawn } = require('child_process');
 const { finished } = require('internal/streams/end-of-stream');
-const { resolve } = require('path');
+const { resolve, sep, isAbsolute } = require('path');
 const { DefaultDeserializer, DefaultSerializer } = require('v8');
 const { getOptionValue } = require('internal/options');
 const { Interface } = require('internal/readline/interface');
@@ -55,13 +57,13 @@ const {
   validateObject,
   validateOneOf,
   validateInteger,
+  validateString,
+  validateStringArray,
 } = require('internal/validators');
 const { getInspectPort, isUsingInspector, isInspectorMessage } = require('internal/util/inspector');
 const { isRegExp } = require('internal/util/types');
 const { pathToFileURL } = require('internal/url');
 const {
-  createDeferredPromise,
-  getCWDURL,
   kEmptyObject,
 } = require('internal/util');
 const { kEmitMessage } = require('internal/test_runner/tests_stream');
@@ -129,7 +131,14 @@ function filterExecArgv(arg, i, arr) {
   !ArrayPrototypeSome(kFilterArgValues, (p) => arg === p || (i > 0 && arr[i - 1] === p) || StringPrototypeStartsWith(arg, `${p}=`));
 }
 
-function getRunArgs(path, { forceExit, inspectPort, testNamePatterns, testSkipPatterns, only }) {
+function getRunArgs(path, { forceExit,
+                            inspectPort,
+                            testNamePatterns,
+                            testSkipPatterns,
+                            only,
+                            argv: suppliedArgs,
+                            execArgv,
+                            cwd }) {
   const argv = ArrayPrototypeFilter(process.execArgv, filterExecArgv);
   if (forceExit === true) {
     ArrayPrototypePush(argv, '--test-force-exit');
@@ -147,12 +156,17 @@ function getRunArgs(path, { forceExit, inspectPort, testNamePatterns, testSkipPa
     ArrayPrototypePush(argv, '--test-only');
   }
 
+  ArrayPrototypePushApply(argv, execArgv);
+
   if (path === kIsolatedProcessName) {
-    ArrayPrototypePush(argv, '--test', ...ArrayPrototypeSlice(process.argv, 1));
+    ArrayPrototypePush(argv, '--test');
+    ArrayPrototypePushApply(argv, ArrayPrototypeSlice(process.argv, 1));
   } else {
     ArrayPrototypePush(argv, path);
   }
 
+  ArrayPrototypePushApply(argv, suppliedArgs);
+
   return argv;
 }
 
@@ -274,7 +288,7 @@ class FileTest extends Test {
       ArrayPrototypePush(this.#rawBuffer, readData);
     }
     this.#rawBufferSize += dataLength;
-    this.#proccessRawBuffer();
+    this.#processRawBuffer();
 
     if (partialV8Header) {
       ArrayPrototypePush(this.#rawBuffer, TypedArrayPrototypeSubarray(v8Header, 0, 1));
@@ -283,10 +297,10 @@ class FileTest extends Test {
   }
   #drainRawBuffer() {
     while (this.#rawBuffer.length > 0) {
-      this.#proccessRawBuffer();
+      this.#processRawBuffer();
     }
   }
-  #proccessRawBuffer() {
+  #processRawBuffer() {
     // This method is called when it is known that there is at least one message
     let bufferHead = this.#rawBuffer[0];
     let headerIndex = bufferHead.indexOf(v8Header);
@@ -481,7 +495,8 @@ function watchFiles(testFiles, opts) {
       // When file renamed (created / deleted) we need to update the watcher
       if (newFileName) {
         owners = new SafeSet().add(newFileName);
-        watcher.filterFile(resolve(newFileName), owners);
+        const resolveFileName = isAbsolute(newFileName) ? newFileName : resolve(opts.cwd, newFileName);
+        watcher.filterFile(resolveFileName, owners);
       }
 
       if (!newFileName && previousFileName) {
@@ -524,7 +539,13 @@ function watchFiles(testFiles, opts) {
 function run(options = kEmptyObject) {
   validateObject(options, 'options');
 
-  let { testNamePatterns, testSkipPatterns, shard } = options;
+  let {
+    testNamePatterns,
+    testSkipPatterns,
+    shard,
+    coverageExcludeGlobs,
+    coverageIncludeGlobs,
+  } = options;
   const {
     concurrency,
     timeout,
@@ -537,6 +558,13 @@ function run(options = kEmptyObject) {
     setup,
     only,
     globPatterns,
+    coverage = false,
+    lineCoverage = 0,
+    branchCoverage = 0,
+    functionCoverage = 0,
+    execArgv = [],
+    argv = [],
+    cwd = process.cwd(),
   } = options;
 
   if (files != null) {
@@ -561,6 +589,8 @@ function run(options = kEmptyObject) {
     validateArray(globPatterns, 'options.globPatterns');
   }
 
+  validateString(cwd, 'options.cwd');
+
   if (globPatterns?.length > 0 && files?.length > 0) {
     throw new ERR_INVALID_ARG_VALUE(
       'options.globPatterns', globPatterns, 'is not supported when specifying \'options.files\'',
@@ -615,6 +645,25 @@ function run(options = kEmptyObject) {
     });
   }
   validateOneOf(isolation, 'options.isolation', ['process', 'none']);
+  validateBoolean(coverage, 'options.coverage');
+  if (coverageExcludeGlobs != null) {
+    if (!ArrayIsArray(coverageExcludeGlobs)) {
+      coverageExcludeGlobs = [coverageExcludeGlobs];
+    }
+    validateStringArray(coverageExcludeGlobs, 'options.coverageExcludeGlobs');
+  }
+  if (coverageIncludeGlobs != null) {
+    if (!ArrayIsArray(coverageIncludeGlobs)) {
+      coverageIncludeGlobs = [coverageIncludeGlobs];
+    }
+    validateStringArray(coverageIncludeGlobs, 'options.coverageIncludeGlobs');
+  }
+  validateInteger(lineCoverage, 'options.lineCoverage', 0, 100);
+  validateInteger(branchCoverage, 'options.branchCoverage', 0, 100);
+  validateInteger(functionCoverage, 'options.functionCoverage', 0, 100);
+
+  validateStringArray(argv, 'options.argv');
+  validateStringArray(execArgv, 'options.execArgv');
 
   const rootTestOptions = { __proto__: null, concurrency, timeout, signal };
   const globalOptions = {
@@ -623,12 +672,15 @@ function run(options = kEmptyObject) {
     // behavior has relied on it, so removing it must be done in a semver major.
     ...parseCommandLine(),
     setup,  // This line can be removed when parseCommandLine() is removed here.
+    coverage,
+    coverageExcludeGlobs,
+    coverageIncludeGlobs,
+    lineCoverage: lineCoverage,
+    branchCoverage: branchCoverage,
+    functionCoverage: functionCoverage,
+    cwd,
   };
   const root = createTestTree(rootTestOptions, globalOptions);
-
-  // This const should be replaced by a run option in the future.
-  const cwd = process.cwd();
-
   let testFiles = files ?? createTestFileList(globPatterns, cwd);
 
   if (shard) {
@@ -652,6 +704,8 @@ function run(options = kEmptyObject) {
     forceExit,
     cwd,
     isolation,
+    argv,
+    execArgv,
   };
 
   if (isolation === 'process') {
@@ -679,7 +733,8 @@ function run(options = kEmptyObject) {
     };
   } else if (isolation === 'none') {
     if (watch) {
-      filesWatcher = watchFiles(testFiles, opts);
+      const absoluteTestFiles = ArrayPrototypeMap(testFiles, (file) => (isAbsolute(file) ? file : resolve(cwd, file)));
+      filesWatcher = watchFiles(absoluteTestFiles, opts);
       runFiles = async () => {
         root.harness.bootstrapPromise = null;
         root.harness.buildPromise = null;
@@ -689,10 +744,10 @@ function run(options = kEmptyObject) {
       };
     } else {
       runFiles = async () => {
-        const { promise, resolve: finishBootstrap } = createDeferredPromise();
+        const { promise, resolve: finishBootstrap } = PromiseWithResolvers();
 
         await root.runInAsyncScope(async () => {
-          const parentURL = getCWDURL().href;
+          const parentURL = pathToFileURL(cwd + sep).href;
           const cascadedLoader = esmLoader.getOrInitializeCascadedLoader();
           let topLevelTestCount = 0;
 
@@ -705,7 +760,7 @@ function run(options = kEmptyObject) {
 
           for (let i = 0; i < testFiles.length; ++i) {
             const testFile = testFiles[i];
-            const fileURL = pathToFileURL(testFile);
+            const fileURL = pathToFileURL(resolve(cwd, testFile));
             const parent = i === 0 ? undefined : parentURL;
             let threw = false;
             let importError;
