@@ -1,8 +1,10 @@
 'use strict';
 const {
   ArrayPrototypeFlatMap,
+  ArrayPrototypeForEach,
   ArrayPrototypeJoin,
   ArrayPrototypeMap,
+  ArrayPrototypePop,
   ArrayPrototypePush,
   ArrayPrototypeReduce,
   ArrayPrototypeSome,
@@ -12,6 +14,7 @@ const {
   NumberParseInt,
   NumberPrototypeToFixed,
   ObjectGetOwnPropertyDescriptor,
+  PromiseWithResolvers,
   RegExp,
   RegExpPrototypeExec,
   SafeMap,
@@ -24,10 +27,9 @@ const {
 } = primordials;
 
 const { AsyncResource } = require('async_hooks');
-const { relative } = require('path');
+const { relative, sep } = require('path');
 const { createWriteStream } = require('fs');
 const { pathToFileURL } = require('internal/url');
-const { createDeferredPromise } = require('internal/util');
 const { getOptionValue } = require('internal/options');
 const { green, yellow, red, white, shouldColorize } = require('internal/util/colors');
 
@@ -52,12 +54,15 @@ const kMultipleCallbackInvocations = 'multipleCallbackInvocations';
 const kRegExpPattern = /^\/(.*)\/([a-z]*)$/;
 
 const kPatterns = ['test', 'test/**/*', 'test-*', '*[._-]test'];
-const kDefaultPattern = `**/{${ArrayPrototypeJoin(kPatterns, ',')}}.?(c|m)js`;
-
+const kFileExtensions = ['js', 'mjs', 'cjs'];
+if (getOptionValue('--experimental-strip-types')) {
+  ArrayPrototypePush(kFileExtensions, 'ts', 'mts', 'cts');
+}
+const kDefaultPattern = `**/{${ArrayPrototypeJoin(kPatterns, ',')}}.{${ArrayPrototypeJoin(kFileExtensions, ',')}}`;
 
 function createDeferredCallback() {
   let calledCount = 0;
-  const { promise, resolve, reject } = createDeferredPromise();
+  const { promise, resolve, reject } = PromiseWithResolvers();
   const cb = (err) => {
     calledCount++;
 
@@ -119,7 +124,7 @@ const kBuiltinReporters = new SafeMap([
   ['lcov', 'internal/test_runner/reporter/lcov'],
 ]);
 
-const kDefaultReporter = process.stdout.isTTY ? 'spec' : 'tap';
+const kDefaultReporter = 'spec';
 const kDefaultDestination = 'stdout';
 
 function tryBuiltinReporter(name) {
@@ -142,7 +147,8 @@ function shouldColorizeTestFiles(destinations) {
 
 async function getReportersMap(reporters, destinations) {
   return SafePromiseAllReturnArrayLike(reporters, async (name, i) => {
-    const destination = kBuiltinDestinations.get(destinations[i]) ?? createWriteStream(destinations[i]);
+    const destination = kBuiltinDestinations.get(destinations[i]) ??
+      createWriteStream(destinations[i], { __proto__: null, flush: true });
 
     // Load the test reporter passed to --test-reporter
     let reporter = tryBuiltinReporter(name);
@@ -299,6 +305,8 @@ function parseCommandLine() {
       const { reporter, destination } = reportersMap[i];
       compose(rootReporter, reporter).pipe(destination);
     }
+
+    reporterScope.reporters = reportersMap;
   });
 
   globalTestOptions = {
@@ -355,12 +363,14 @@ function countCompletedTest(test, harness = test.root.harness) {
     harness.counters.todo++;
   } else if (test.cancelled) {
     harness.counters.cancelled++;
+    harness.success = false;
   } else if (!test.passed) {
     harness.counters.failed++;
+    harness.success = false;
   } else {
     harness.counters.passed++;
   }
-  harness.counters.all++;
+  harness.counters.tests++;
 }
 
 
@@ -409,6 +419,34 @@ const kColumns = ['line %', 'branch %', 'funcs %'];
 const kColumnsKeys = ['coveredLinePercent', 'coveredBranchPercent', 'coveredFunctionPercent'];
 const kSeparator = ' | ';
 
+function buildFileTree(summary) {
+  const tree = { __proto__: null };
+  let treeDepth = 1;
+  let longestFile = 0;
+
+  ArrayPrototypeForEach(summary.files, (file) => {
+    let longestPart = 0;
+    const parts = StringPrototypeSplit(relative(summary.workingDirectory, file.path), sep);
+    let current = tree;
+
+    ArrayPrototypeForEach(parts, (part, index) => {
+      current[part] ||= { __proto__: null };
+      current = current[part];
+      // If this is the last part, add the file to the tree
+      if (index === parts.length - 1) {
+        current.file = file;
+      }
+      // Keep track of the longest part for padding
+      longestPart = MathMax(longestPart, part.length);
+    });
+
+    treeDepth = MathMax(treeDepth, parts.length);
+    longestFile = MathMax(longestPart, longestFile);
+  });
+
+  return { __proto__: null, tree, treeDepth, longestFile };
+}
+
 function getCoverageReport(pad, summary, symbol, color, table) {
   const prefix = `${pad}${symbol}`;
   let report = `${color}${prefix}start of coverage report\n`;
@@ -418,11 +456,19 @@ function getCoverageReport(pad, summary, symbol, color, table) {
   let uncoveredLinesPadLength;
   let tableWidth;
 
+  // Create a tree of file paths
+  const { tree, treeDepth, longestFile } = buildFileTree(summary);
   if (table) {
-    // Get expected column sizes
-    filePadLength = table && ArrayPrototypeReduce(summary.files, (acc, file) =>
-      MathMax(acc, relative(summary.workingDirectory, file.path).length), 0);
+    // Calculate expected column sizes based on the tree
+    filePadLength = table && longestFile;
+    filePadLength += (treeDepth - 1);
+    if (color) {
+      filePadLength += 2;
+    }
     filePadLength = MathMax(filePadLength, 'file'.length);
+    if (filePadLength > (process.stdout.columns / 2)) {
+      filePadLength = MathFloor(process.stdout.columns / 2);
+    }
     const fileWidth = filePadLength + 2;
 
     columnPadLengths = ArrayPrototypeMap(kColumns, (column) => (table ? MathMax(column.length, 6) : 0));
@@ -435,26 +481,17 @@ function getCoverageReport(pad, summary, symbol, color, table) {
 
     tableWidth = fileWidth + columnsWidth + uncoveredLinesWidth;
 
-    // Fit with sensible defaults
     const availableWidth = (process.stdout.columns || Infinity) - prefix.length;
     const columnsExtras = tableWidth - availableWidth;
     if (table && columnsExtras > 0) {
-      // Ensure file name is sufficiently visible
-      const minFilePad = MathMin(8, filePadLength);
-      filePadLength -= MathFloor(columnsExtras * 0.2);
-      filePadLength = MathMax(filePadLength, minFilePad);
-
-      // Get rest of available space, subtracting margins
+      filePadLength = MathMin(availableWidth * 0.5, filePadLength);
       uncoveredLinesPadLength = MathMax(availableWidth - columnsWidth - (filePadLength + 2) - 2, 1);
-
-      // Update table width
       tableWidth = availableWidth;
     } else {
       uncoveredLinesPadLength = Infinity;
     }
   }
 
-
   function getCell(string, width, pad, truncate, coverage) {
     if (!table) return string;
 
@@ -469,35 +506,85 @@ function getCoverageReport(pad, summary, symbol, color, table) {
     return result;
   }
 
-  // Head
-  if (table) report += addTableLine(prefix, tableWidth);
-  report += `${prefix}${getCell('file', filePadLength, StringPrototypePadEnd, truncateEnd)}${kSeparator}` +
-            `${ArrayPrototypeJoin(ArrayPrototypeMap(kColumns, (column, i) => getCell(column, columnPadLengths[i], StringPrototypePadStart)), kSeparator)}${kSeparator}` +
-            `${getCell('uncovered lines', uncoveredLinesPadLength, false, truncateEnd)}\n`;
-  if (table) report += addTableLine(prefix, tableWidth);
+  function writeReportLine({ file, depth = 0, coveragesColumns, fileCoverage, uncoveredLines }) {
+    const fileColumn = `${prefix}${StringPrototypeRepeat(' ', depth)}${getCell(file, filePadLength - depth, StringPrototypePadEnd, truncateStart, fileCoverage)}`;
+    const coverageColumns = ArrayPrototypeJoin(ArrayPrototypeMap(coveragesColumns, (coverage, j) => {
+      const coverageText = typeof coverage === 'number' ? NumberPrototypeToFixed(coverage, 2) : coverage;
+      return getCell(coverageText, columnPadLengths[j], StringPrototypePadStart, false, coverage);
+    }), kSeparator);
 
-  // Body
-  for (let i = 0; i < summary.files.length; ++i) {
-    const file = summary.files[i];
-    const relativePath = relative(summary.workingDirectory, file.path);
+    const uncoveredLinesColumn = getCell(uncoveredLines, uncoveredLinesPadLength, false, truncateEnd);
 
-    let fileCoverage = 0;
-    const coverages = ArrayPrototypeMap(kColumnsKeys, (columnKey) => {
-      const percent = file[columnKey];
-      fileCoverage += percent;
-      return percent;
-    });
-    fileCoverage /= kColumnsKeys.length;
-
-    report += `${prefix}${getCell(relativePath, filePadLength, StringPrototypePadEnd, truncateStart, fileCoverage)}${kSeparator}` +
-              `${ArrayPrototypeJoin(ArrayPrototypeMap(coverages, (coverage, j) => getCell(NumberPrototypeToFixed(coverage, 2), columnPadLengths[j], StringPrototypePadStart, false, coverage)), kSeparator)}${kSeparator}` +
-              `${getCell(formatUncoveredLines(getUncoveredLines(file.lines), table), uncoveredLinesPadLength, false, truncateEnd)}\n`;
+    return `${fileColumn}${kSeparator}${coverageColumns}${kSeparator}${uncoveredLinesColumn}\n`;
   }
 
-  // Foot
+  function printCoverageBodyTree(tree, depth = 0) {
+    for (const key in tree) {
+      if (tree[key].file) {
+        const file = tree[key].file;
+        const fileName = ArrayPrototypePop(StringPrototypeSplit(file.path, sep));
+
+        let fileCoverage = 0;
+        const coverages = ArrayPrototypeMap(kColumnsKeys, (columnKey) => {
+          const percent = file[columnKey];
+          fileCoverage += percent;
+          return percent;
+        });
+        fileCoverage /= kColumnsKeys.length;
+
+        const uncoveredLines = formatUncoveredLines(getUncoveredLines(file.lines), table);
+
+        report += writeReportLine({
+          __proto__: null,
+          file: fileName,
+          depth: depth,
+          coveragesColumns: coverages,
+          fileCoverage: fileCoverage,
+          uncoveredLines: uncoveredLines,
+        });
+      } else {
+        report += writeReportLine({
+          __proto__: null,
+          file: key,
+          depth: depth,
+          coveragesColumns: ArrayPrototypeMap(columnPadLengths, () => ''),
+          fileCoverage: undefined,
+          uncoveredLines: '',
+        });
+        printCoverageBodyTree(tree[key], depth + 1);
+      }
+    }
+  }
+
+  // -------------------------- Coverage Report --------------------------
   if (table) report += addTableLine(prefix, tableWidth);
-  report += `${prefix}${getCell('all files', filePadLength, StringPrototypePadEnd, truncateEnd)}${kSeparator}` +
-            `${ArrayPrototypeJoin(ArrayPrototypeMap(kColumnsKeys, (columnKey, j) => getCell(NumberPrototypeToFixed(summary.totals[columnKey], 2), columnPadLengths[j], StringPrototypePadStart, false, summary.totals[columnKey])), kSeparator)} |\n`;
+
+  // Print the header
+  report += writeReportLine({
+    __proto__: null,
+    file: 'file',
+    coveragesColumns: kColumns,
+    fileCoverage: undefined,
+    uncoveredLines: 'uncovered lines',
+  });
+
+  if (table) report += addTableLine(prefix, tableWidth);
+
+  // Print the body
+  printCoverageBodyTree(tree);
+
+  if (table) report += addTableLine(prefix, tableWidth);
+
+  // Print the footer
+  const allFilesCoverages = ArrayPrototypeMap(kColumnsKeys, (columnKey) => summary.totals[columnKey]);
+  report += writeReportLine({
+    __proto__: null,
+    file: 'all files',
+    coveragesColumns: allFilesCoverages,
+    fileCoverage: undefined,
+    uncoveredLines: '',
+  });
+
   if (table) report += addTableLine(prefix, tableWidth);
 
   report += `${prefix}end of coverage report\n`;
