@@ -2,7 +2,6 @@
 
 const {
   ArrayFrom,
-  ArrayIsArray,
   ArrayPrototypePush,
   ArrayPrototypeSlice,
   ArrayPrototypeSort,
@@ -34,9 +33,7 @@ const {
   SafeSet,
   SafeWeakMap,
   SafeWeakRef,
-  StringPrototypeIncludes,
   StringPrototypeReplace,
-  StringPrototypeStartsWith,
   StringPrototypeToLowerCase,
   StringPrototypeToUpperCase,
   Symbol,
@@ -48,6 +45,7 @@ const {
 const {
   codes: {
     ERR_NO_CRYPTO,
+    ERR_NO_TYPESCRIPT,
     ERR_UNKNOWN_SIGNAL,
   },
   isErrorStackTraceLimitWritable,
@@ -68,6 +66,7 @@ const { getOptionValue } = require('internal/options');
 const { encodings } = internalBinding('string_decoder');
 
 const noCrypto = !process.versions.openssl;
+const noTypeScript = !process.versions.amaro;
 
 const isWindows = process.platform === 'win32';
 const isMacOS = process.platform === 'darwin';
@@ -197,9 +196,17 @@ function assertCrypto() {
     throw new ERR_NO_CRYPTO();
 }
 
-// Return undefined if there is no match.
-// Move the "slow cases" to a separate function to make sure this function gets
-// inlined properly. That prioritizes the common case.
+function assertTypeScript() {
+  if (noTypeScript)
+    throw new ERR_NO_TYPESCRIPT();
+}
+
+/**
+ * Move the "slow cases" to a separate function to make sure this function gets
+ * inlined properly. That prioritizes the common case.
+ * @param {unknown} enc
+ * @returns {string | undefined} Returns undefined if there is no match.
+ */
 function normalizeEncoding(enc) {
   if (enc == null || enc === 'utf8' || enc === 'utf-8') return 'utf8';
   return slowCases(enc);
@@ -259,10 +266,13 @@ function slowCases(enc) {
   }
 }
 
-function emitExperimentalWarning(feature) {
+function emitExperimentalWarning(feature, messagePrefix) {
   if (experimentalWarnings.has(feature)) return;
-  const msg = `${feature} is an experimental feature and might change at any time`;
   experimentalWarnings.add(feature);
+  let msg = `${feature} is an experimental feature and might change at any time`;
+  if (messagePrefix) {
+    msg = messagePrefix + msg;
+  }
   process.emitWarning(msg, 'ExperimentalWarning');
 }
 
@@ -379,6 +389,10 @@ function getCWDURL() {
   return cachedURL;
 }
 
+function getSystemErrorMessage(err) {
+  return lazyUv().getErrorMessage(err);
+}
+
 function getSystemErrorName(err) {
   const entry = uvErrmapGet(err);
   return entry ? entry[0] : `Unknown system error ${err}`;
@@ -481,6 +495,10 @@ function spliceOne(list, index) {
 
 const kNodeModulesRE = /^(?:.*)[\\/]node_modules[\\/]/;
 
+function isUnderNodeModules(filename) {
+  return filename && (RegExpPrototypeExec(kNodeModulesRE, filename) !== null);
+}
+
 let getStructuredStackImpl;
 
 function lazyGetStructuredStack() {
@@ -509,31 +527,6 @@ function getStructuredStack() {
   return getStructuredStackImpl();
 }
 
-function isInsideNodeModules() {
-  const stack = getStructuredStack();
-
-  // Iterate over all stack frames and look for the first one not coming
-  // from inside Node.js itself:
-  if (ArrayIsArray(stack)) {
-    for (const frame of stack) {
-      const filename = frame.getFileName();
-
-      if (
-        filename == null ||
-        StringPrototypeStartsWith(filename, 'node:') === true ||
-        (
-          filename[0] !== '/' &&
-          StringPrototypeIncludes(filename, '\\') === false
-        )
-      ) {
-        continue;
-      }
-      return RegExpPrototypeExec(kNodeModulesRE, filename) !== null;
-    }
-  }
-  return false;
-}
-
 function once(callback, { preserveReturnValue = false } = kEmptyObject) {
   let called = false;
   let returnValue;
@@ -557,17 +550,6 @@ function sleep(msec) {
   _sleep(msec);
 }
 
-function createDeferredPromise() {
-  let resolve;
-  let reject;
-  const promise = new Promise((res, rej) => {
-    resolve = res;
-    reject = rej;
-  });
-
-  return { promise, resolve, reject };
-}
-
 // https://heycam.github.io/webidl/#define-the-operations
 function defineOperation(target, name, method) {
   ObjectDefineProperty(target, name, {
@@ -768,9 +750,9 @@ function SideEffectFreeRegExpPrototypeExec(regex, string) {
   return FunctionPrototypeCall(RegExpFromAnotherRealm.prototype.exec, regex, string);
 }
 
-const crossRelmRegexes = new SafeWeakMap();
-function getCrossRelmRegex(regex) {
-  const cached = crossRelmRegexes.get(regex);
+const crossRealmRegexes = new SafeWeakMap();
+function getCrossRealmRegex(regex) {
+  const cached = crossRealmRegexes.get(regex);
   if (cached) return cached;
 
   let flagString = '';
@@ -783,17 +765,17 @@ function getCrossRelmRegex(regex) {
   if (RegExpPrototypeGetSticky(regex)) flagString += 'y';
 
   const { RegExp: RegExpFromAnotherRealm } = getInternalGlobal();
-  const crossRelmRegex = new RegExpFromAnotherRealm(RegExpPrototypeGetSource(regex), flagString);
-  crossRelmRegexes.set(regex, crossRelmRegex);
-  return crossRelmRegex;
+  const crossRealmRegex = new RegExpFromAnotherRealm(RegExpPrototypeGetSource(regex), flagString);
+  crossRealmRegexes.set(regex, crossRealmRegex);
+  return crossRealmRegex;
 }
 
 function SideEffectFreeRegExpPrototypeSymbolReplace(regex, string, replacement) {
-  return getCrossRelmRegex(regex)[SymbolReplace](string, replacement);
+  return getCrossRealmRegex(regex)[SymbolReplace](string, replacement);
 }
 
 function SideEffectFreeRegExpPrototypeSymbolSplit(regex, string, limit = undefined) {
-  return getCrossRelmRegex(regex)[SymbolSplit](string, limit);
+  return getCrossRealmRegex(regex)[SymbolSplit](string, limit);
 }
 
 /**
@@ -882,10 +864,10 @@ for (let i = 0; i < encodings.length; ++i)
 module.exports = {
   getLazy,
   assertCrypto,
+  assertTypeScript,
   cachedResult,
   convertToValidSignal,
   createClassWrapper,
-  createDeferredPromise,
   decorateErrorStack,
   defineOperation,
   defineLazyProperties,
@@ -905,9 +887,10 @@ module.exports = {
   getStructuredStack,
   getSystemErrorMap,
   getSystemErrorName,
+  getSystemErrorMessage,
   guessHandleType,
   isError,
-  isInsideNodeModules,
+  isUnderNodeModules,
   isMacOS,
   isWindows,
   join,
@@ -924,14 +907,6 @@ module.exports = {
   setupCoverageHooks,
   removeColors,
 
-  // Define Symbol.dispose and Symbol.asyncDispose
-  // Until these are defined by the environment.
-  // TODO(MoLow): Remove this polyfill once Symbol.dispose and Symbol.asyncDispose are available in primordials.
-  // eslint-disable-next-line node-core/prefer-primordials
-  SymbolDispose: Symbol.dispose || SymbolFor('nodejs.dispose'),
-  // eslint-disable-next-line node-core/prefer-primordials
-  SymbolAsyncDispose: Symbol.asyncDispose || SymbolFor('nodejs.asyncDispose'),
-
   // Symbol used to customize promisify conversion
   customPromisifyArgs: kCustomPromisifyArgsSymbol,
 
