@@ -27,6 +27,7 @@ const {
 
 const { compare } = internalBinding('buffer');
 const assert = require('internal/assert');
+const { isURL } = require('internal/url');
 const types = require('internal/util/types');
 const {
   isAnyArrayBuffer,
@@ -46,6 +47,8 @@ const {
   isFloat64Array,
   isKeyObject,
   isCryptoKey,
+  isWeakMap,
+  isWeakSet,
 } = types;
 const {
   constants: {
@@ -283,7 +286,14 @@ function innerDeepEqual(val1, val2, strict, memos) {
     ) {
       return false;
     }
+  } else if (isWeakMap(val1) || isWeakSet(val1)) {
+    return false;
+  } else if (isURL(val1)) {
+    if (!isURL(val2) || val1.href !== val2.href) {
+      return false;
+    }
   }
+
   return keyCheck(val1, val2, strict, memos, kNoIterator);
 }
 
