@@ -12,6 +12,7 @@ const {
   NumberMAX_SAFE_INTEGER,
   NumberMIN_SAFE_INTEGER,
   ObjectAssign,
+  ObjectPrototypeIsPrototypeOf,
   SafeSet,
   String,
   SymbolIterator,
@@ -20,6 +21,7 @@ const {
 
 const {
   codes: {
+    ERR_INVALID_ARG_TYPE,
     ERR_INVALID_ARG_VALUE,
   },
 } = require('internal/errors');
@@ -36,6 +38,16 @@ converters.any = (V) => {
   return V;
 };
 
+converters.object = (V, opts = kEmptyObject) => {
+  if (type(V) !== 'Object') {
+    throw makeException(
+      'is not an object',
+      kEmptyObject,
+    );
+  }
+  return V;
+};
+
 // https://webidl.spec.whatwg.org/#abstract-opdef-integerpart
 const integerPart = MathTrunc;
 
@@ -187,6 +199,8 @@ converters.DOMString = function DOMString(V) {
   return String(V);
 };
 
+converters['sequence<object>'] = createSequenceConverter(converters.object);
+
 function codedTypeError(message, errorProperties = kEmptyObject) {
   // eslint-disable-next-line no-restricted-syntax
   const err = new TypeError(message);
@@ -241,7 +255,7 @@ function type(V) {
     case 'object': // Fall through
     case 'function': // Fall through
     default:
-      // Per ES spec, typeof returns an implemention-defined value that is not
+      // Per ES spec, typeof returns an implementation-defined value that is not
       // any of the existing ones for uncallable non-standard exotic objects.
       // Yet Type() which the Web IDL spec depends on returns Object for such
       // cases. So treat the default case as an object.
@@ -275,7 +289,7 @@ function createSequenceConverter(converter) {
       const val = converter(res.value, {
         __proto__: null,
         ...opts,
-        context: `${opts.context}, index ${array.length}`,
+        context: `${opts.context}[${array.length}]`,
       });
       ArrayPrototypePush(array, val);
     };
@@ -283,12 +297,26 @@ function createSequenceConverter(converter) {
   };
 }
 
+// https://webidl.spec.whatwg.org/#js-interface
+function createInterfaceConverter(name, I) {
+  return (V, opts = kEmptyObject) => {
+    // 1. If V implements I, then return the IDL interface type value that
+    //    represents a reference to that platform object.
+    if (ObjectPrototypeIsPrototypeOf(I, V)) return V;
+    // 2. Throw a TypeError.
+    throw new ERR_INVALID_ARG_TYPE(
+      typeof opts.context === 'string' ? opts.context : 'value', name, V,
+    );
+  };
+}
+
 
 module.exports = {
   type,
   converters,
   convertToInt,
   createEnumConverter,
+  createInterfaceConverter,
   createSequenceConverter,
   evenRound,
   makeException,
