@@ -30,7 +30,7 @@ const {
   decodeURIComponent,
 } = primordials;
 
-const { toASCII } = require('internal/idna');
+const { toASCII } = internalBinding('encoding_binding');
 const { encodeStr, hexTable } = require('internal/querystring');
 const querystring = require('querystring');
 
@@ -790,8 +790,8 @@ Url.prototype.resolveObject = function resolveObject(relative) {
         !hostlessProtocol.has(relative.protocol)) {
       const relPath = (relative.pathname || '').split('/');
       while (relPath.length && !(relative.host = relPath.shift()));
-      if (!relative.host) relative.host = '';
-      if (!relative.hostname) relative.hostname = '';
+      relative.host ||= '';
+      relative.hostname ||= '';
       if (relPath[0] !== '') relPath.unshift('');
       if (relPath.length < 2) relPath.unshift('');
       result.pathname = relPath.join('/');
@@ -810,7 +810,7 @@ Url.prototype.resolveObject = function resolveObject(relative) {
       const s = result.search || '';
       result.path = p + s;
     }
-    result.slashes = result.slashes || relative.slashes;
+    result.slashes ||= relative.slashes;
     result.href = result.format();
     return result;
   }
@@ -850,7 +850,7 @@ Url.prototype.resolveObject = function resolveObject(relative) {
       }
       relative.host = null;
     }
-    mustEndAbs = mustEndAbs && (relPath[0] === '' || srcPath[0] === '');
+    mustEndAbs &&= (relPath[0] === '' || srcPath[0] === '');
   }
 
   if (isRelAbs) {
@@ -871,7 +871,7 @@ Url.prototype.resolveObject = function resolveObject(relative) {
   } else if (relPath.length) {
     // it's relative
     // throw away the existing file, and take the new path instead.
-    if (!srcPath) srcPath = [];
+    srcPath ||= [];
     srcPath.pop();
     srcPath = srcPath.concat(relPath);
     result.search = relative.search;
@@ -974,7 +974,7 @@ Url.prototype.resolveObject = function resolveObject(relative) {
     }
   }
 
-  mustEndAbs = mustEndAbs || (result.host && srcPath.length);
+  mustEndAbs ||= (result.host && srcPath.length);
 
   if (mustEndAbs && !isAbsolute) {
     srcPath.unshift('');
@@ -993,7 +993,7 @@ Url.prototype.resolveObject = function resolveObject(relative) {
                   (result.search ? result.search : '');
   }
   result.auth = relative.auth || result.auth;
-  result.slashes = result.slashes || relative.slashes;
+  result.slashes ||= relative.slashes;
   result.href = result.format();
   return result;
 };
