@@ -289,11 +289,11 @@ const BindingData::PackageConfig* BindingData::TraverseParent(
 
     // Stop the search when the process doesn't have permissions
     // to walk upwards
-    if (UNLIKELY(is_permissions_enabled &&
-                 !env->permission()->is_granted(
-                     env,
-                     permission::PermissionScope::kFileSystemRead,
-                     current_path.generic_string()))) {
+    if (is_permissions_enabled &&
+        !env->permission()->is_granted(
+            env,
+            permission::PermissionScope::kFileSystemRead,
+            current_path.generic_string())) [[unlikely]] {
       return nullptr;
     }
 
@@ -322,14 +322,13 @@ void BindingData::GetNearestParentPackageJSON(
   BufferValue path_value(realm->isolate(), args[0]);
   // Check if the path has a trailing slash. If so, add it after
   // ToNamespacedPath() as it will be deleted by ToNamespacedPath()
-  bool slashCheck = path_value.ToStringView().ends_with(
-      std::filesystem::path::preferred_separator);
+  bool slashCheck = path_value.ToStringView().ends_with(kPathSeparator);
 
   ToNamespacedPath(realm->env(), &path_value);
 
   std::string path_value_str = path_value.ToString();
   if (slashCheck) {
-    path_value_str.push_back(std::filesystem::path::preferred_separator);
+    path_value_str.push_back(kPathSeparator);
   }
 
   auto package_json =
@@ -349,14 +348,13 @@ void BindingData::GetNearestParentPackageJSONType(
   BufferValue path_value(realm->isolate(), args[0]);
   // Check if the path has a trailing slash. If so, add it after
   // ToNamespacedPath() as it will be deleted by ToNamespacedPath()
-  bool slashCheck = path_value.ToStringView().ends_with(
-      std::filesystem::path::preferred_separator);
+  bool slashCheck = path_value.ToStringView().ends_with(kPathSeparator);
 
   ToNamespacedPath(realm->env(), &path_value);
 
   std::string path_value_str = path_value.ToString();
   if (slashCheck) {
-    path_value_str.push_back(std::filesystem::path::preferred_separator);
+    path_value_str.push_back(kPathSeparator);
   }
 
   auto package_json =
@@ -435,6 +433,55 @@ void BindingData::GetPackageScopeConfig(
           .ToLocalChecked());
 }
 
+void FlushCompileCache(const FunctionCallbackInfo<Value>& args) {
+  Isolate* isolate = args.GetIsolate();
+  Local<Context> context = isolate->GetCurrentContext();
+  Environment* env = Environment::GetCurrent(context);
+
+  if (!args[0]->IsBoolean() && !args[0]->IsUndefined()) {
+    THROW_ERR_INVALID_ARG_TYPE(env,
+                               "keepDeserializedCache should be a boolean");
+    return;
+  }
+  Debug(env,
+        DebugCategory::COMPILE_CACHE,
+        "[compile cache] module.flushCompileCache() requested.\n");
+  env->FlushCompileCache();
+  Debug(env,
+        DebugCategory::COMPILE_CACHE,
+        "[compile cache] module.flushCompileCache() finished.\n");
+}
+
+void EnableCompileCache(const FunctionCallbackInfo<Value>& args) {
+  Isolate* isolate = args.GetIsolate();
+  Local<Context> context = isolate->GetCurrentContext();
+  Environment* env = Environment::GetCurrent(context);
+  if (!args[0]->IsString()) {
+    THROW_ERR_INVALID_ARG_TYPE(env, "cacheDir should be a string");
+    return;
+  }
+  Utf8Value value(isolate, args[0]);
+  CompileCacheEnableResult result = env->EnableCompileCache(*value);
+  std::vector<Local<Value>> values = {
+      v8::Integer::New(isolate, static_cast<uint8_t>(result.status)),
+      ToV8Value(context, result.message).ToLocalChecked(),
+      ToV8Value(context, result.cache_directory).ToLocalChecked()};
+  args.GetReturnValue().Set(Array::New(isolate, values.data(), values.size()));
+}
+
+void GetCompileCacheDir(const FunctionCallbackInfo<Value>& args) {
+  Isolate* isolate = args.GetIsolate();
+  Local<Context> context = isolate->GetCurrentContext();
+  Environment* env = Environment::GetCurrent(context);
+  if (!env->use_compile_cache()) {
+    args.GetReturnValue().Set(v8::String::Empty(isolate));
+    return;
+  }
+  args.GetReturnValue().Set(
+      ToV8Value(context, env->compile_cache_handler()->cache_dir())
+          .ToLocalChecked());
+}
+
 void BindingData::CreatePerIsolateProperties(IsolateData* isolate_data,
                                              Local<ObjectTemplate> target) {
   Isolate* isolate = isolate_data->isolate();
@@ -448,6 +495,9 @@ void BindingData::CreatePerIsolateProperties(IsolateData* isolate_data,
             "getNearestParentPackageJSON",
             GetNearestParentPackageJSON);
   SetMethod(isolate, target, "getPackageScopeConfig", GetPackageScopeConfig);
+  SetMethod(isolate, target, "enableCompileCache", EnableCompileCache);
+  SetMethod(isolate, target, "getCompileCacheDir", GetCompileCacheDir);
+  SetMethod(isolate, target, "flushCompileCache", FlushCompileCache);
 }
 
 void BindingData::CreatePerContextProperties(Local<Object> target,
@@ -456,6 +506,20 @@ void BindingData::CreatePerContextProperties(Local<Object> target,
                                              void* priv) {
   Realm* realm = Realm::GetCurrent(context);
   realm->AddBindingData<BindingData>(target);
+
+  std::vector<Local<Value>> compile_cache_status_values;
+  Isolate* isolate = context->GetIsolate();
+
+#define V(status)                                                              \
+  compile_cache_status_values.push_back(                                       \
+      FIXED_ONE_BYTE_STRING(isolate, #status));
+  COMPILE_CACHE_STATUS(V)
+
+  USE(target->Set(context,
+                  FIXED_ONE_BYTE_STRING(isolate, "compileCacheStatus"),
+                  Array::New(isolate,
+                             compile_cache_status_values.data(),
+                             compile_cache_status_values.size())));
 }
 
 void BindingData::RegisterExternalReferences(
@@ -464,6 +528,9 @@ void BindingData::RegisterExternalReferences(
   registry->Register(GetNearestParentPackageJSONType);
   registry->Register(GetNearestParentPackageJSON);
   registry->Register(GetPackageScopeConfig);
+  registry->Register(EnableCompileCache);
+  registry->Register(GetCompileCacheDir);
+  registry->Register(FlushCompileCache);
 }
 
 }  // namespace modules
