@@ -1,6 +1,7 @@
 #include "node_url.h"
 #include "ada.h"
 #include "base_object-inl.h"
+#include "node_debug.h"
 #include "node_errors.h"
 #include "node_external_reference.h"
 #include "node_i18n.h"
@@ -173,12 +174,14 @@ void BindingData::CanParse(const FunctionCallbackInfo<Value>& args) {
 
 bool BindingData::FastCanParse(Local<Value> receiver,
                                const FastOneByteString& input) {
+  TRACK_V8_FAST_API_CALL("url.canParse");
   return ada::can_parse(std::string_view(input.data, input.length));
 }
 
 bool BindingData::FastCanParseWithBase(Local<Value> receiver,
                                        const FastOneByteString& input,
                                        const FastOneByteString& base) {
+  TRACK_V8_FAST_API_CALL("url.canParse.withBase");
   auto base_view = std::string_view(base.data, base.length);
   return ada::can_parse(std::string_view(input.data, input.length), &base_view);
 }
