@@ -41,6 +41,7 @@ using v8::Int32;
 using v8::Integer;
 using v8::Isolate;
 using v8::Just;
+using v8::JustVoid;
 using v8::Local;
 using v8::Maybe;
 using v8::MaybeLocal;
@@ -441,7 +442,7 @@ MaybeLocal<Object> SyncProcessRunner::Run(Local<Value> options) {
 
   CHECK_EQ(lifecycle_, kUninitialized);
 
-  Maybe<bool> r = TryInitializeAndRunLoop(options);
+  Maybe<void> r = TryInitializeAndRunLoop(options);
   CloseHandlesAndDeleteLoop();
   if (r.IsNothing()) return MaybeLocal<Object>();
 
@@ -450,7 +451,7 @@ MaybeLocal<Object> SyncProcessRunner::Run(Local<Value> options) {
   return scope.Escape(result);
 }
 
-Maybe<bool> SyncProcessRunner::TryInitializeAndRunLoop(Local<Value> options) {
+Maybe<void> SyncProcessRunner::TryInitializeAndRunLoop(Local<Value> options) {
   int r;
 
   // There is no recovery from failure inside TryInitializeAndRunLoop - the
@@ -461,7 +462,7 @@ Maybe<bool> SyncProcessRunner::TryInitializeAndRunLoop(Local<Value> options) {
   uv_loop_ = new uv_loop_t;
   if (uv_loop_ == nullptr) {
     SetError(UV_ENOMEM);
-    return Just(false);
+    return JustVoid();
   }
 
   r = uv_loop_init(uv_loop_);
@@ -469,21 +470,21 @@ Maybe<bool> SyncProcessRunner::TryInitializeAndRunLoop(Local<Value> options) {
     delete uv_loop_;
     uv_loop_ = nullptr;
     SetError(r);
-    return Just(false);
+    return JustVoid();
   }
 
-  if (!ParseOptions(options).To(&r)) return Nothing<bool>();
+  if (!ParseOptions(options).To(&r)) return Nothing<void>();
 
   if (r < 0) {
     SetError(r);
-    return Just(false);
+    return JustVoid();
   }
 
   if (timeout_ > 0) {
     r = uv_timer_init(uv_loop_, &uv_timer_);
     if (r < 0) {
       SetError(r);
-      return Just(false);
+      return JustVoid();
     }
 
     uv_unref(reinterpret_cast<uv_handle_t*>(&uv_timer_));
@@ -498,7 +499,7 @@ Maybe<bool> SyncProcessRunner::TryInitializeAndRunLoop(Local<Value> options) {
     r = uv_timer_start(&uv_timer_, KillTimerCallback, timeout_, 0);
     if (r < 0) {
       SetError(r);
-      return Just(false);
+      return JustVoid();
     }
   }
 
@@ -506,7 +507,7 @@ Maybe<bool> SyncProcessRunner::TryInitializeAndRunLoop(Local<Value> options) {
   r = uv_spawn(uv_loop_, &uv_process_, &uv_process_options_);
   if (r < 0) {
     SetError(r);
-    return Just(false);
+    return JustVoid();
   }
   uv_process_.data = this;
 
@@ -515,7 +516,7 @@ Maybe<bool> SyncProcessRunner::TryInitializeAndRunLoop(Local<Value> options) {
       r = pipe->Start();
       if (r < 0) {
         SetPipeError(r);
-        return Just(false);
+        return JustVoid();
       }
     }
   }
@@ -527,10 +528,9 @@ Maybe<bool> SyncProcessRunner::TryInitializeAndRunLoop(Local<Value> options) {
 
   // If we get here the process should have exited.
   CHECK_GE(exit_status_, 0);
-  return Just(true);
+  return JustVoid();
 }
 
-
 void SyncProcessRunner::CloseHandlesAndDeleteLoop() {
   CHECK_LT(lifecycle_, kHandlesClosed);
 
