@@ -247,7 +247,7 @@ T* UncheckedRealloc(T* pointer, size_t n) {
 
   void* allocated = realloc(pointer, full_size);
 
-  if (UNLIKELY(allocated == nullptr)) {
+  if (allocated == nullptr) [[unlikely]] {
     // Tell V8 that memory is low and retry.
     LowMemoryNotification();
     allocated = realloc(pointer, full_size);
@@ -326,7 +326,7 @@ v8::MaybeLocal<v8::Value> ToV8Value(v8::Local<v8::Context> context,
                                     std::string_view str,
                                     v8::Isolate* isolate) {
   if (isolate == nullptr) isolate = context->GetIsolate();
-  if (UNLIKELY(str.size() >= static_cast<size_t>(v8::String::kMaxLength))) {
+  if (str.size() >= static_cast<size_t>(v8::String::kMaxLength)) [[unlikely]] {
     // V8 only has a TODO comment about adding an exception when the maximum
     // string size is exceeded.
     ThrowErrStringTooLong(isolate);
